package cologne.eck.all_peas.cloud.gui;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2020  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Desktop;
import java.awt.Font;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.security.SecureRandom;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import javax.swing.Action;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.SwingWorker;
import javax.swing.WindowConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.DefaultEditorKit;

import org.bouncycastle.crypto.BlockCipher;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.CloudPropertyHandler;
import cologne.eck.all_peas.cloud.StorageProvider;
//import cologne.eck.all_peas.cloud.StorageProvider;
import cologne.eck.all_peas.cloud.provider.WebDavProvider;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.IconManager;
import cologne.eck.all_peas.gui.PeaDialog;
//import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.all_peas.gui.PeaLockFrame;
import cologne.eck.tools.Comparator;
import cologne.eck.tools.Converter;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.Zeroizer;


@SuppressWarnings("serial")
public class WebDavLogin extends JDialog 
	implements ActionListener, DocumentListener, FocusListener, WindowListener {
	
	private WebDavLogin login;
	
	/**
	 * SwingWorker for authorization process
	 */
	private SwingWorker<String, Void> sw;
	/**
	 * Possible values: 0 success, 1 error, -1 close
	 */
	private static int result = 0;
	
	/**
	 * the providers with passwords stored from open dialog to store
	 * later as encrypted property
	 */
	private static HashMap <String, byte[]>  providerPswMap = new HashMap<String, byte[]>();
	/**
	 * The cipher used for encryption and decryption of the cloud passwords
	 */
	private static BlockCipher usedCipher = null;
	/**
	 * the temporary key to xor with the stored password
	 */
	private static byte[] tempKey = null;
	private WebDavProvider provider;
	private Component owner;
	
	//private static boolean lastAuthSuccess = true;
	
	private JLabel titleLabel;
	private String title;
	
	// the combo box for cloud providers
	private JComboBox<String> combo;
	
	// info and link to create an account:
	private JPanel websitePanel;
	
	// checkbox for manual input
	private JCheckBox manualCheck;
	
	// panel that holds serverField
	private JPanel serverNamePanel;
	// contains the server address to construct a WebDAV base path for some cloud software (Nextcloud, Owncloud, Seafile)
	// this is not the WebDAV base path to use!
	private JTextField serverField;
	// set notes for the server address
	private JLabel serverNoteLabel;
	
	// the WebDAV base path of the cloud provider
	private JTextField webDavBaseField;
	// notes for the WebDAV base path
	private JLabel webDavNoteLabel;
	// one more help text for webDavBaseField
	private JLabel webDavHelpLabel;

	// the name of the provider
	private JTextField providerField;
	
	// server software: Nextcloud,Owncloud, Seafile
	private JPanel radioButtonPanel1;
	private JPanel radioButtonPanel2;
	
	private JPanel webDavBasePanel;
	
	// The user name to login
	private JTextField userNameField;
	// notes for the user name
	private JLabel userNameNoteLabel;
	
	// the password to login
	private JPasswordField passwordField;
	// save the password or not
	//private JCheckBox passwordCheck;
	private JLabel passwordNoteLabel;
	
	private JPopupMenu cutCopyPasteMenu;
	// the language bundle to use)
	private ResourceBundle languageBundle;
	
	// the cloud software as String: Nextcloud, Owncloud, Seafile, Unknown
	private String providerSoftware = "Unknown";

	// the font used for buttons, labels...
	private static Font font = new Font(Font.SANS_SERIF, Font.PLAIN, 12);
	// the border to display an error
	private static LineBorder errorBorder = new LineBorder( Color.RED, 2);
	
	//private static final TreeMap<String, String> providerPaths = new TreeMap<String, String>() {{
		
	private static final LinkedHashMap<String, List<String>> providerPaths = new LinkedHashMap<String, List<String>>() {{

		put(PeaProperties.getVmBridge().translate("choose") + "...", null);
		put("   Open Source: ", null);
		//String[] good = {"https://use02.thegood.cloud/remote.php/webdav", "https://thegood.cloud/"};
		put("The Good Cloud (Nextcloud)", Arrays.asList("https://use02.thegood.cloud/remote.php/webdav", "https://thegood.cloud/"));
		//String[] oCloud = { "https://***YOUR_CLOUD_NAME***.oCloud.de/owncloud/remote.php/webdav", "https://ocloud.de/"};
		put("oCloud (Owncloud)", Arrays.asList("https://***YOUR_CLOUD_NAME***.oCloud.de/owncloud/remote.php/webdav", "https://ocloud.de/"));
		//String[] ops = { };
		put("OpsData (Nextcloud)",  Arrays.asList("https://cloud01.opsdata.ch/remote.php/webdav", "https://opsone.ch/en/managed-applications/nextcloud"));
		//String[] ownDrive = { "https://my.owndrive.com/remote.php/webdav", "https://owndrive.com/"};
		put("OwnDrive (Nextcloud)", Arrays.asList("https://my.owndrive.com/remote.php/webdav", "https://owndrive.com/"));
		//String[] tab = {  "https://c.tab.digital/remote.php/webdav", "https://cloud.tab.digital/"};
		put("Tab.Digital (Nextcloud)",  Arrays.asList("https://c.tab.digital/remote.php/webdav", "https://cloud.tab.digital/"));
		//String[] wint = { "https://ocloud.global/remote.php/webdav", "https://wint.global/ocloud/"};
		put("Wint.global (Owncloud)", Arrays.asList("https://ocloud.global/remote.php/webdav", "https://wint.global/ocloud/"));
		//String[] woelkli = { "https://cloud.woelkli.com/remote.php/webdav", "https://woelkli.com/en"};
		put("Wölkli (Nextcloud)",  Arrays.asList("https://cloud.woelkli.com/remote.php/webdav", "https://woelkli.com/en"));
		put( "______________________", null);
		put("1&1 (DSL)", Arrays.asList("https://sd2dav.1und1.de"));
		put("1&1 (Webhosting)", Arrays.asList("https://webdav.office.1und1.de"));
		put("4shared", Arrays.asList("https://webdav.4shared.com/"));
		//put("Box", "https://dav.box.com/dav");
		put("CloudMe", Arrays.asList("https://webdav.cloudme.com/***YOUR_USER_NAME***"));
		put("Cubby", Arrays.asList("https://webdav.cubby.com"));
		put("DriveOnWeb", Arrays.asList("https://storage.driveonweb.de/probdav"));		
		put("Freenet", Arrays.asList("https://webmail.freenet.de/webdav"));
		put("GMX", Arrays.asList("https://webdav.mc.gmx.net"));		
		put("HiDrive", Arrays.asList("https://webdav.hidrive.strato.com"));
		put("IDrive", Arrays.asList("https://dav.idrivesync.com"));
		put("MagentaCLOUD", Arrays.asList("https://webdav.magentacloud.de"));
		put("Mail.Ru", Arrays.asList("https://webdav.cloud.mail.ru"));
		put("Mailbox.org", Arrays.asList("https://dav.mailbox.org/servlet/webdav.infostore/"));
		put("Memopal", Arrays.asList("https://dav.memopal.com/"));
		put("MyDrive", Arrays.asList("https://webdav.mydrive.ch"));			
		put("OpenDrive", Arrays.asList("https://webdav.opendrive.com/"));		
		put("pCloud", Arrays.asList("https://webdav.pcloud.com"));
		put("PowerFolder", Arrays.asList("https://my.powerfolder.com/webdav"));		
		put("STACK", Arrays.asList("https://{username}.stackstorage.com/remote.php/webdav"));		
		put("Storage Made Easy (EU)", Arrays.asList("https://webdaveu.storagemadeeasy.com"));
		put("Storage Made Easy (US)", Arrays.asList("https://webdav.storagemadeeasy.com"));		
		put("Strato HiDrive", Arrays.asList("https://webdav.hidrive.strato.com"));
		put("Syncwerk", Arrays.asList("https://app.syncwerk.com/seafdav"));		
		put("WEB.DE", Arrays.asList("https://webdav.smartdrive.web.de"));
		put("Yandex.Disk", Arrays.asList("https://webdav.yandex.com"));
		
		}};



	
	/**
	 * Login into a WebDavProvider in a modal dialog
	 * and set properties
	 * 
	 * @param _owner		the window owner
	 * @param _title		the localized title: add, modify or null
	 * @param _provider		the provider or null
	 * @param _providerName		the provider name or null
	 * @param _userName		the user name or null
	 * @param _password		the password as char array or null
	 * @param _webDavBasePath	the WebDAV base path or null
	 */
	public WebDavLogin(Component _owner, String _title, WebDavProvider _provider, 
			String _providerName, String _userName, char[] _password, String _webDavBasePath) {
		
		login = this;
		this.owner = _owner;
		if (_provider == null) {
			//this.provider = new WebDavProvider(_providerName);
			StorageProvider sp = CloudControl.getOpenProvider(_providerName);
			if (sp == null) {
				 sp = new WebDavProvider(_providerName);
			}
			if (sp instanceof WebDavProvider) {
				this.provider = (WebDavProvider) sp;
			}
		} else {
			this.provider = _provider;
		}
		languageBundle = (ResourceBundle) JREProperties.getBundle();		
		this.setModal(true);
		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(this);

		this.setIconImage(PswDialogView.getImage());
		//this.setMinimumSize(new Dimension(500,400));
		//this.provider = _provider;
		boolean modifyModus = false;
		this.title = _title;
		if (title == null) {
			if (_providerName == null) {
				this.setTitle(languageBundle.getString("add_cloud_provider"));
			} else {
				this.setTitle(_providerName);
			}
		} else {
			this.setTitle(title);//languageBundle.getString("add_cloud_provider"));
			if (title.equals(languageBundle.getString("modify_cloud_provider"))) {
				modifyModus = true;
			}
		}
		
		JPanel contentPane = (JPanel) this.getContentPane();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
		contentPane.setBorder(new EmptyBorder(10,10,10,10));
		
		//contentPane.add(Box.createVerticalStrut(10));
		JPanel titleLabelPanel = new JPanel();
		titleLabelPanel.setLayout(new BoxLayout(titleLabelPanel, BoxLayout.LINE_AXIS));
		titleLabel = new JLabel(languageBundle.getString("need_existing_account"));
		titleLabelPanel.add(titleLabel);
		titleLabelPanel.add(Box.createHorizontalGlue());
		

		
		contentPane.add(titleLabelPanel);
		
		websitePanel = new JPanel();
		contentPane.add(websitePanel);
		
		
		contentPane.add(Box.createVerticalStrut(10));

		if (modifyModus == false){
			JPanel providerComboBoxPanel = new JPanel();
			providerComboBoxPanel.setLayout(new BoxLayout(providerComboBoxPanel, BoxLayout.LINE_AXIS));
			JLabel providerComboBoxLabel = new JLabel(languageBundle.getString("choose_provider"));
			providerComboBoxLabel.setFont(font);
			String[] provNames = getProviderNames();
			combo = new JComboBox<String>(provNames);
			//combo.setSelectedItem("OpenDrive");
			//		if ( ! provNames.equals())
			combo.addActionListener(this);
			combo.setActionCommand("PROVIDER");
			providerComboBoxPanel.add(Box.createHorizontalGlue());
			providerComboBoxPanel.add(providerComboBoxLabel);
			providerComboBoxPanel.add(combo);
			providerComboBoxPanel.add(Box.createHorizontalGlue());
			contentPane.add(providerComboBoxPanel);
		}	
		JPanel manuallyPanel = new JPanel();
		manuallyPanel.setLayout(new BoxLayout(manuallyPanel, BoxLayout.LINE_AXIS));
		//		JLabel manuallyLabel = new JLabel(languageBundle.getString("or_enter_manually"));
		manualCheck = new JCheckBox(languageBundle.getString("enter_manually")); 
		manualCheck.setSelected(false);
		manualCheck.addActionListener(this);
		manualCheck.setActionCommand("manual");
		manualCheck.setFont(font);
		manuallyPanel.add(manualCheck);
		manuallyPanel.add(Box.createHorizontalGlue());
		contentPane.add(manuallyPanel);
		contentPane.add(Box.createVerticalStrut(10));


		// cut-copy-paste functionality for text fields:
		cutCopyPasteMenu = new JPopupMenu();
		Action cut = new DefaultEditorKit.CutAction();
        cut.putValue(Action.NAME, languageBundle.getString("cut"));
        cut.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control X"));
        cutCopyPasteMenu.add( cut );
        Action copy = new DefaultEditorKit.CopyAction();
        copy.putValue(Action.NAME, languageBundle.getString("copy"));
        copy.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control C"));
        cutCopyPasteMenu.add( copy );
        Action paste = new DefaultEditorKit.PasteAction();
        paste.putValue(Action.NAME, languageBundle.getString("paste"));
        paste.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("control V"));
        cutCopyPasteMenu.add( paste );
        // for password field
        JPopupMenu pasteOnly = new JPopupMenu();
        pasteOnly.add(paste);

		
		JPanel providerNamePanel = new JPanel();
		providerNamePanel.setLayout(new BoxLayout(providerNamePanel, BoxLayout.PAGE_AXIS));
		JLabel providerNameLabel = new JLabel(languageBundle.getString("enter_provider_name"));
		providerNameLabel.setFont(font);
		JPanel providerLabelPanel = new JPanel();
		providerLabelPanel.setLayout(new BoxLayout(providerLabelPanel, BoxLayout.LINE_AXIS));
		providerLabelPanel.add(providerNameLabel);
		providerLabelPanel.add(Box.createHorizontalGlue());
		providerNamePanel.add(providerLabelPanel);
		providerField = new JTextField();
		if (_providerName != null) {
			providerField.setText(_providerName);
		} else {
			if (provider != null && provider.getProviderName() != null) {
				providerField.setText(provider.getProviderName());
			} 
		}
		providerField.getDocument().putProperty("owner", providerField);
		providerField.getDocument().addDocumentListener(this);
		providerField.setComponentPopupMenu( cutCopyPasteMenu );
		JPanel providerFieldPanel = new JPanel();
		providerFieldPanel.setLayout((new BoxLayout(providerFieldPanel, BoxLayout.LINE_AXIS)));
		providerFieldPanel.add(providerField);
		providerFieldPanel.add(Box.createHorizontalGlue());
		providerNamePanel.add(providerFieldPanel);
		providerField.setToolTipText(languageBundle.getString("user_defined"));		
		contentPane.add(providerNamePanel);
		contentPane.add(Box.createVerticalStrut(10));
		if (modifyModus == true) {
			providerField.setEditable(false);
		}
		
		
		radioButtonPanel1 = new JPanel();
		radioButtonPanel1.setLayout(new BoxLayout(radioButtonPanel1, BoxLayout.LINE_AXIS));
		JLabel providerSoftwareLabel = new JLabel(languageBundle.getString("server_runs_with"));
		providerSoftwareLabel.setFont(font);
		radioButtonPanel1.add(providerSoftwareLabel);
		
	    JRadioButton unknownButton = new JRadioButton(languageBundle.getString("no_idea"));
	    unknownButton.setFont(font);
	    unknownButton.setActionCommand("Unknown");
	    unknownButton.setSelected(true);
		
		JRadioButton nextcloudButton = new JRadioButton("Nextcloud");
		nextcloudButton.setFont(font);
	    nextcloudButton.setMnemonic(KeyEvent.VK_N);
	    nextcloudButton.setActionCommand("Nextcloud");

	    JRadioButton owncloudButton = new JRadioButton("Owncloud");
	    owncloudButton.setFont(font);
	    owncloudButton.setMnemonic(KeyEvent.VK_O);
	    owncloudButton.setActionCommand("Owncloud");

	    JRadioButton seafileButton = new JRadioButton("Seafile");
	    seafileButton.setFont(font);
	    seafileButton.setMnemonic(KeyEvent.VK_S);
	    seafileButton.setActionCommand("Seafile");

	    //Group the radio buttons.
	    ButtonGroup group = new ButtonGroup();
	    group.add(nextcloudButton);
	    group.add(owncloudButton);
	    group.add(seafileButton);
	    group.add(unknownButton);

	    //Register a listener for the radio buttons.
	    nextcloudButton.addActionListener(this);
	    owncloudButton.addActionListener(this);
	    seafileButton.addActionListener(this);
	    unknownButton.addActionListener(this);
	    
	    radioButtonPanel2 = new JPanel();
	    radioButtonPanel2.setLayout(new BoxLayout(radioButtonPanel2, BoxLayout.LINE_AXIS));
	    radioButtonPanel1.add(unknownButton);
	    radioButtonPanel1.add(Box.createHorizontalGlue());
	    radioButtonPanel2.add(Box.createHorizontalStrut(20));
	    radioButtonPanel2.add(nextcloudButton);
	    radioButtonPanel2.add(owncloudButton);
	    radioButtonPanel2.add(seafileButton);
	    radioButtonPanel2.add(Box.createHorizontalGlue());
		contentPane.add(radioButtonPanel1);
		contentPane.add(radioButtonPanel2);
		contentPane.add(Box.createVerticalStrut(20));
		
		
		JPanel userNamePanel = new JPanel();
		userNamePanel.setLayout(new BoxLayout(userNamePanel, BoxLayout.PAGE_AXIS));
		JLabel userNameLabel = new JLabel(languageBundle.getString("enter_user_name"));
		userNameLabel.setFont(font);
		JPanel userNameLabelPanel = new JPanel();
		userNameLabelPanel.setLayout(new BoxLayout(userNameLabelPanel, BoxLayout.LINE_AXIS));
		userNameLabelPanel.add(userNameLabel);
		userNameLabelPanel.add(Box.createHorizontalGlue());
		userNamePanel.add(userNameLabelPanel);
		userNameField = new JTextField();
		if (_userName != null) {
			userNameField.setText(_userName);
		}
		userNameField.getDocument().putProperty("owner", userNameField);
		userNameField.getDocument().addDocumentListener(this);
		userNameField.setComponentPopupMenu( cutCopyPasteMenu );
		JPanel userNameFieldPanel = new JPanel();
		userNameFieldPanel.setLayout((new BoxLayout(userNameFieldPanel, BoxLayout.LINE_AXIS)));
		userNameFieldPanel.add(userNameField);
		userNameFieldPanel.add(Box.createHorizontalGlue());
		userNamePanel.add(userNameFieldPanel);
		userNameField.setToolTipText(languageBundle.getString("try_user_name_or_mail_address"));
		userNameNoteLabel = new JLabel("");
		userNameNoteLabel.setFont(font);
		JPanel userNameNoteLabelPanel = new JPanel();
		userNameNoteLabelPanel.setLayout(new BoxLayout(userNameNoteLabelPanel, BoxLayout.LINE_AXIS));
		userNameNoteLabelPanel.add(userNameNoteLabel);
		userNameNoteLabelPanel.add(Box.createHorizontalGlue());
		userNamePanel.add(userNameNoteLabelPanel);
		contentPane.add(userNamePanel);
		contentPane.add(Box.createVerticalStrut(20));
		
		JPanel passwordPanel = new JPanel();
		passwordPanel.setLayout(new BoxLayout(passwordPanel, BoxLayout.PAGE_AXIS));
		JLabel passwordLabel = new JLabel(languageBundle.getString("enter_cloud_password") );
		passwordLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		JPanel passwordLabelPanel = new JPanel();
		passwordLabelPanel.setLayout(new BoxLayout(passwordLabelPanel, BoxLayout.LINE_AXIS));
		passwordLabelPanel.add(passwordLabel);
		passwordLabelPanel.add(Box.createHorizontalGlue());
		passwordPanel.add(passwordLabelPanel);
		JPanel passwordFieldPanel = new JPanel();
		passwordFieldPanel.setLayout(new BoxLayout(passwordFieldPanel, BoxLayout.LINE_AXIS));
		passwordField = new JPasswordField();
		if (_password != null) {
			passwordField.setText(new String(_password));
		}
		passwordField.getDocument().putProperty("owner", passwordField);
		passwordField.getDocument().addDocumentListener(this);
		passwordField.setComponentPopupMenu( pasteOnly );
		passwordFieldPanel.add(passwordField);
		JCheckBox passwordHideCheck = new JCheckBox();
		passwordHideCheck.addActionListener(this);
		passwordHideCheck.setActionCommand("passwordHide");
		passwordFieldPanel.add(passwordHideCheck);
		JLabel eyeLabel = new JLabel();
		eyeLabel.setIcon(IconManager.loadIcon("eye.png", "eye",  "eye", "hide or show the password"));
		passwordFieldPanel.add(eyeLabel);
		passwordFieldPanel.add(Box.createHorizontalGlue());
		passwordPanel.add(passwordFieldPanel);
		
		passwordNoteLabel = new JLabel(languageBundle.getString("is_stored_encrypted"));
		passwordNoteLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		passwordPanel.add(passwordNoteLabel);

		contentPane.add(passwordPanel);
		contentPane.add(Box.createVerticalStrut(20));		

		webDavBasePanel = new JPanel();
		webDavBasePanel.setLayout(new BoxLayout(webDavBasePanel, BoxLayout.PAGE_AXIS));
		JPanel webDavBaseLabelPanel = new JPanel();
		webDavBaseLabelPanel.setLayout(new BoxLayout(webDavBaseLabelPanel, BoxLayout.LINE_AXIS));
		JLabel webDavBaseLabel = new JLabel(languageBundle.getString("enter_web_dav_path"));
		webDavBaseLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		JButton helpButton = new JButton(languageBundle.getString("help"));
		helpButton.setFont(font);
		helpButton.addActionListener(this);
		helpButton.setActionCommand("help");
		webDavBaseLabelPanel.add(webDavBaseLabel);
		webDavBaseLabelPanel.add(Box.createHorizontalGlue());
		webDavBaseLabelPanel.add(helpButton);
		webDavBasePanel.add(webDavBaseLabelPanel);
		webDavBaseField = new JTextField();
		if (_webDavBasePath != null) {
			webDavBaseField.setText(_webDavBasePath);
		} else {
			webDavBaseField.setText("https://webdav.example.org");
			webDavBaseField.setForeground(Color.GRAY);
		}
		JPanel webDavNoteLabelPanel = new JPanel();
		webDavNoteLabelPanel.setLayout(new BoxLayout(webDavNoteLabelPanel, BoxLayout.LINE_AXIS));
		webDavNoteLabel = new JLabel("");//Note: Generally the server publishes the base path");
		webDavNoteLabel.setFont(new Font(Font.SANS_SERIF, Font.ITALIC, 12));
		webDavNoteLabelPanel.add(webDavNoteLabel);
		webDavNoteLabelPanel.add(Box.createHorizontalGlue());
		webDavBaseField.getDocument().addDocumentListener(this);
		webDavBaseField.setComponentPopupMenu( cutCopyPasteMenu );
		webDavBaseField.getDocument().putProperty("owner", webDavBaseField);
		webDavBaseField.addFocusListener(this);

		JPanel webDavBaseFieldPanel = new JPanel();
		webDavBaseFieldPanel.setLayout(new BoxLayout(webDavBaseFieldPanel, BoxLayout.LINE_AXIS));
		webDavBaseFieldPanel.add(webDavBaseField);
		webDavBaseFieldPanel.add(Box.createHorizontalGlue());
		webDavBasePanel.add(webDavBaseFieldPanel);
		webDavBaseField.setToolTipText(languageBundle.getString("provider_publish_web_dav_path"));
		webDavBasePanel.add(webDavNoteLabelPanel);
		webDavBasePanel.add(Box.createVerticalStrut(10));
		
		JPanel webDavHelpLabelPanel = new JPanel();
		webDavHelpLabelPanel.setLayout(new BoxLayout(webDavHelpLabelPanel, BoxLayout.LINE_AXIS));
		webDavHelpLabel = new JLabel("");//Note: Generally the server publishes the base path");
		webDavHelpLabel.setFont(new Font(Font.SANS_SERIF, Font.ITALIC, 12));
		webDavHelpLabelPanel.add(webDavHelpLabel);
		webDavHelpLabelPanel.add(Box.createHorizontalGlue());
		webDavBasePanel.add(webDavHelpLabelPanel);
		
		contentPane.add(webDavBasePanel);
		//contentPane.add(Box.createVerticalStrut(20));
		
		serverNamePanel = new JPanel();
		serverNamePanel.setLayout(new BoxLayout(serverNamePanel, BoxLayout.PAGE_AXIS));	
		//webDavBaseLabel.setAlignmentX(RIGHT_ALIGNMENT);
		webDavBasePanel.add(serverNamePanel);

		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel,BoxLayout.LINE_AXIS));
		JButton okButton = new JButton("OK");
		okButton.addActionListener(this);
		okButton.setActionCommand("ok");
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(okButton);
		contentPane.add(buttonPanel);
		
		radioButtonPanel1.setVisible(false);
		radioButtonPanel2.setVisible(false);
		webDavBasePanel.setVisible(false);
		
		this.pack();
		this.setLocationRelativeTo(_owner);
	}

	/**
	 * The WebDavLogin must have been created before calling this function.
	 * WebDavLogin should always be called by this method
	 * to get a resulting value
	 * 
	 * @return	result of login process: 
	 * 			-1 cancelled, 0 success, 1 failed
	 */
	public int showLoginDialog() {//Component _owner, WebDavProvider _provider, 
			//String _providerName, String _userName, char[] _password, String _webDavBasePath) {
		// Close all existing error dialogs:
		Window[] windows = Window.getWindows();
        for (Window window : windows) {
            if (window instanceof JDialog) {
                JDialog dialog = (JDialog) window;
                if (dialog.getContentPane().getComponentCount() > 0
                    && dialog.getContentPane().getComponent(0) instanceof JOptionPane){
                    dialog.dispose();
                }
            }
        }
        //login = new WebDavLogin( _owner, _provider, 
		//	 _providerName, _userName, _password, _webDavBasePath);
	    login.setVisible(true);
	    return result;
	}

	// TEST:
/*	public static void main(String[] args) {	
		WebDavLogin wdl = new WebDavLogin(null, new WebDavProvider(null, null), null, null, null, null);
		wdl.setVisible(true);
	} */
	
	/**
	 * Get all available provider names
	 * 
	 * @return	provider names of WebDAV providers
	 */
	private static String[] getProviderNames() {
		return providerPaths.keySet().toArray(new String[providerPaths.size()]);
	}

	/**
	 * Get the WebDAV base path of a provider
	 * 
	 * @param providerName	the name of the provider
	 * 
	 * @return	the WebDAV base path of this provider
	 */
	private static String getBasePath(String providerName) {
		List<String> l = providerPaths.get(providerName);
		if (l != null && l.size() > 0) {
			return providerPaths.get(providerName).get(0);
		} else {
			TestLog.ve(WebDavLogin.class, "Invalid list of provider " + providerName);
			return null;
		}
	}
	
	/**
	 * Get the web site to create an account of a provider
	 * 
	 * @param providerName	the name of the provider
	 * 
	 * @return	the web site of this provider
	 */
	private static String getWebsite(String providerName) {
		List<String> l = providerPaths.get(providerName);
		if (l == null) {
			TestLog.ve(WebDavLogin.class, "Invalid list of provider " + providerName);
			return null;
		}
		int listSize = l.size();
		if (listSize == 0) {
			TestLog.ve(WebDavLogin.class, "Empty list of provider " + providerName);
			return null;
		} else if (listSize == 1) { // closed source providers
			return null;
		} else if (listSize > 1) {
			return l.get(1);
		} else {
			TestLog.ve(WebDavLogin.class, "Invalid list of provider " + providerName);
			return null;
		}
		//return providerPaths.get(providerName).get(1);
	}
	
/*		// To use for web site
	public static void main(String[] args) {	 		
		for (String name: providerPaths.keySet()){
			String key = name.toString();
			String value = providerPaths.get(name).toString();  
			TestLog.v(WebDavLogin.class, "<li><strong>" + key + "</strong>: " + value + "</li>");  
		} 
	} */
	
	/**
	 * Set the title label for the login form
	 * 
	 * @param _newLabelText
	 */
/*	public void setTitleLabel(String _newLabelText, boolean isError) {
		titleLabel.setText(_newLabelText);
		if (isError == true) {
			titleLabel.setForeground(Color.RED);
		} else {
			titleLabel.setForeground(Color.BLACK);
		}
	} */

	/**
	 * Get all cloud login password from PEA open dialog
	 * as password properties in the cloud property file. 
	 * The password are encrypted by a temporary key and must
	 * be encrypted by the session key, derived from the
	 * PEA password
	 * 
	 * @return	a HashMap with providerName and decrypted keys
	 * 			as byte array
	 */
	public static HashMap<String, byte[]> getPasswordsFromNewProviderDialog() {
		if (providerPswMap.isEmpty()) {
			return null;
		}
		if (tempKey == null) { // this should never happen
			new UnexpectedValueException("tempKey", "byte[]", "is null").printDescription();
			return null;
		} else {
			// check for program bug
			if (PeaProperties.isTestMode()){
				Comparator.checkNullVector(tempKey);
			}
		}
		HashMap <String, byte[]>  plainProviderPswMap = new HashMap<String, byte[]>();
		// iterate over map and set all stored passwords as properties with session key
		// get current cipher to reset later
		BlockCipher lastCipher = CipherStuff.getCipherAlgo();
		if (usedCipher != null) {
			CipherStuff.setCipherAlgo(usedCipher);
			for (Iterator<Map.Entry<String, byte[]>> entries = providerPswMap.entrySet().iterator(); entries.hasNext(); ) {
				Map.Entry<String, byte[]> entry = entries.next();
				String provider = entry.getKey();
				byte[] pswBytes = entry.getValue();
				// decrypt by stored temporary key
				byte[] plainPswBytes = CipherStuff.processCTR(false, pswBytes, tempKey, false);
				plainProviderPswMap.put(provider, plainPswBytes);
			}
			// reset cipher:
			CipherStuff.setCipherAlgo(lastCipher);
		} else {
			TestLog.e(WebDavLogin.class, "Missing block cipher for decryption", 5);
		}
		return plainProviderPswMap;
	}
	/**
	 * Get a manually added and temporary stored password
	 * for a provider
	 * 
	 * @param providerName	the name of the provider
	 * 
	 * @return	the plain text password as array of bytes
	 */
	static byte[] getTemporaryStoredPassword(String providerName) {
		if (providerPswMap.isEmpty()) {
			return null;
		}
		if (providerName == null) {
			TestLog.e(WebDavLogin.class, "Unexpected error");
			TestLog.ve(WebDavLogin.class, "Missing provider name", 5);
			return null;
		}
		byte[] encryptedPswBytes = providerPswMap.get(providerName);
		if (encryptedPswBytes != null) {
			// get current cipher to reset later
			BlockCipher lastCipher = CipherStuff.getCipherAlgo();
			byte[] plainPswBytes = CipherStuff.processCTR(false, encryptedPswBytes, tempKey, false);
			// reset cipher:
			CipherStuff.setCipherAlgo(lastCipher);
			return plainPswBytes;
		} else {
			TestLog.v(WebDavLogin.class, "No temporary stored password for " + providerName);
			return null;
		}		
	}
	
	/**
	 * Store a manually typed password for a provider in encrypted form. 
	 * The password is zeroized.
	 * 
	 * @param providerName		the name of the provider
	 * @param plainPswBytes		the plain password as byte array
	 * 
	 * @return	true if there was no password for this provider before, 
	 * 			false if an already existed password was overwritten
	 */
	public static synchronized boolean storePassword(String providerName, byte[] plainPswBytes) {
		// get current cipher to reset later
		BlockCipher lastCipher = CipherStuff.getCipherAlgo();
		if (usedCipher == null) {
			// cipher may change:
			usedCipher = CipherStuff.getCipherAlgo();
		} else {
			// set this cipher:
			CipherStuff.setCipherAlgo(usedCipher);
		}
		if (tempKey == null) {
			SecureRandom sc = new SecureRandom();
			tempKey = new byte[CipherStuff.getKeySize()];//.getCipherAlgo().getBlockSize()];
			sc.nextBytes(tempKey);
		}						
		byte[] pswBytes = CipherStuff.processCTR(true, plainPswBytes, tempKey, false);
		Zeroizer.zero(plainPswBytes);
		// reset cipher:
		CipherStuff.setCipherAlgo(lastCipher);		
		TestLog.v(WebDavLogin.class, "Store manually typed password in encrypted form for " + providerName);
		byte[] previousKey = providerPswMap.put(providerName, pswBytes);
		if (previousKey == null) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Get the last used WebDavProvider
	 * 
	 * @return	the last used provider
	 */
	public StorageProvider getProvider() {
		return provider;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equals("ok")) {

			sw = new WebDavLoginWorker();
			sw.execute();
			
		} else if (command.equals("manual")) {
			if (manualCheck.isSelected()) {			
				radioButtonPanel1.setVisible(true);
				radioButtonPanel2.setVisible(true);
				webDavBasePanel.setVisible(true);
				this.pack();
			} else {
				radioButtonPanel1.setVisible(false);
				radioButtonPanel2.setVisible(false);
				webDavBasePanel.setVisible(false);
				this.pack();
			}
			
		} else if (command.equals("passwordHide")) {

			JCheckBox check = (JCheckBox) e.getSource();
			if (check.isSelected() == true) {
				passwordField.setEchoChar((char) 0);
			} else {
				passwordField.setEchoChar('*');
			}
		} else if (command.equals("help")) {
			if (serverNamePanel.getComponentCount() < 2) {
				serverNamePanel.setBorder(new EmptyBorder(0, 20, 0,20));
				webDavNoteLabel.setText(languageBundle.getString("provider_publish_web_dav_path"));
				//webDavNoteLabel.setForeground(Color.RED);
				
				if (providerSoftware.equals("Unknown")) { // no further help available...
					this.pack();
					return;
				} else {

					serverNamePanel.setLayout(new BoxLayout(serverNamePanel, BoxLayout.PAGE_AXIS));
					webDavHelpLabel.setText(languageBundle.getString("not_found"));
					JLabel serverLabel = new JLabel(languageBundle.getString("enter_cloud_server_address") );
					serverLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
					JPanel serverLabelPanel = new JPanel();
					serverLabelPanel.setLayout(new BoxLayout(serverLabelPanel, BoxLayout.LINE_AXIS));
					serverLabelPanel.add(Box.createHorizontalStrut(20));
					serverLabelPanel.add(serverLabel);
					serverLabelPanel.add(Box.createHorizontalGlue());
					serverNamePanel.add(serverLabelPanel);
					serverField = new JTextField();
					serverField.setText("https//:example.org");
					serverField.setForeground(Color.GRAY);
					serverField.getDocument().putProperty("owner", serverField);
					serverField.getDocument().addDocumentListener(this);
					serverField.setComponentPopupMenu( cutCopyPasteMenu );
					serverField.addFocusListener(this);
					JPanel serverFieldPanel = new JPanel();
					serverFieldPanel.setLayout(new BoxLayout(serverFieldPanel, BoxLayout.LINE_AXIS));
					serverFieldPanel.add(Box.createHorizontalStrut(20));
					serverFieldPanel.add(serverField);
					serverFieldPanel.add(Box.createHorizontalGlue());
					serverNamePanel.add(serverFieldPanel);
					serverNoteLabel = new JLabel("");
					JPanel serverNotePanel = new JPanel();
					serverNotePanel.setLayout(new BoxLayout(serverNotePanel, BoxLayout.LINE_AXIS));
					serverNotePanel.add(Box.createHorizontalStrut(20));
					serverNotePanel.add(serverNoteLabel);
					serverNotePanel.add(Box.createHorizontalGlue());
					serverNamePanel.add(serverNotePanel);
					serverNamePanel.add(Box.createVerticalStrut(20)); 				
					this.pack();
				}				
			}
			
		} else if (command.startsWith("LINK")){
			
			command = command.substring(4, command.length());
			URI uri = null;
			try {
				uri = new URI(command);
			} catch (URISyntaxException e1) {
				TestLog.e(WebDavLogin.class, "Invalid URI: " + command);
				return;
			}
			if (Desktop.isDesktopSupported() == true) {
				Desktop d = Desktop.getDesktop();
				if (d == null) {
					TestLog.e(WebDavLogin.class, "Invalid Desktop... ");
					return;
				} else {
					try {
						d.browse(uri);
					} catch (IOException e1) {
						TestLog.e(WebDavLogin.class, e1.getLocalizedMessage());
						return;
					}
				}			
				
			} else {
				TestLog.e(WebDavLogin.class, "Desktop is not supported... ");
				return;
			}
			
		} else if (command.equals("PROVIDER")) {
			radioButtonPanel1.setVisible(false);
			radioButtonPanel2.setVisible(false);
			webDavBasePanel.setVisible(false);
			
			String provName = (String) combo.getSelectedItem();
			
			websitePanel.removeAll();
			if (this.title.equals(languageBundle.getString("add_cloud_provider"))) {
				
				// show link to web site to create an account: 
				if (provName != null && getWebsite(provName) != null) {

					websitePanel.setLayout(new BoxLayout(websitePanel, BoxLayout.PAGE_AXIS));
					JPanel sitePanel = new JPanel();

					JLabel createAccountLabel = new JLabel(languageBundle.getString("go_to") );
					sitePanel.add(createAccountLabel);
					//websitePanel.add(createAccountLabel);
					JButton linkButton = new JButton(languageBundle.getString("web"));
					linkButton.addActionListener(this);
					linkButton.setActionCommand("LINK" + getWebsite(provName));
					sitePanel.add(linkButton);
					websitePanel.add(sitePanel);
					JTextField linkField = new JTextField();
					//linkField.setBackground(websitePanel.getBackground());
					linkField.setText(getWebsite(provName));
					linkField.setEditable(false);
					websitePanel.add(linkField);		
					websitePanel.add(Box.createVerticalStrut(5));
					websitePanel.add(new JSeparator(SwingConstants.HORIZONTAL));
					websitePanel.add(Box.createVerticalStrut(15));
				}
			}
			this.pack();
			//TestLog.v(WebDavLogin.class, combo.getSelectedIndex());			

			if (providerPaths.get(provName) == null) {
			//if (provName.equals("   Open Source: ") || provName.equals("______________________")) {
				
			} else {
				//TestLog.v(WebDavLogin.class, "provider name: " + provName);
				webDavBaseField.setText(getBasePath(provName));
			}
		} else if (command.equals("Unknown")) {
			providerSoftware = "Unknown";
		} else if (command.equals("Nextcloud")) {
			providerSoftware = "Nextcloud";
		} else if (command.equals("Owncloud")) {
			providerSoftware = "Owncloud";
		} else if (command.equals("Seafile")) {
			providerSoftware = "Seafile";
		}
	}


	@Override
	public void changedUpdate(DocumentEvent cue) {	}
	
	private void handleDocumentEvents(DocumentEvent iue) {
		// Check serverField and webDavBaseField for valid URL
		// Set webDavBaseField if serverField contains a valid URL
		JTextField comp = (JTextField) iue.getDocument().getProperty("owner");
		if (comp.equals(serverField)) {
			//TestLog.v(WebDavLogin.class, "serverField");
			if ( ! serverField.getText().isEmpty() ) {
				String text = serverField.getText();
				try {
					@SuppressWarnings("unused")
					URL url = new URL(serverField.getText());
				} catch (MalformedURLException e1) {
					//TestLog.e(WebDavLogin.class, "Not a valid URL: " + text);
					serverNoteLabel.setText(languageBundle.getString("enter_valid_web_address"));
					serverNoteLabel.setForeground(Color.RED);
					serverField.setBorder(errorBorder);
					this.pack();
					return;
				}
				serverNoteLabel.setText("");
				serverField.setBorder(null);
				if (text.contains(".") ) {
					// check for index page and remove
					//String basePart = text.substring(0, text.lastIndexOf('.'));
					//TestLog.v(WebDavLogin.class, "basePart: " + basePart);
					if (text.endsWith(".html") || text.endsWith(".htm") 
							|| text.endsWith(".shtml") || text.endsWith(".shtm")
							|| text.endsWith(".php") || text.endsWith(".asp")
							|| text.endsWith(".jsp")) {
						text = text.substring(0, text.lastIndexOf("/"));
						//TestLog.v(WebDavLogin.class, "text: " + text);
					}
				}
				text = text.trim();
				if (! text.endsWith("/")) {
					text = text + "/";
				}
				// Scheme for Nextcloud: requires user name
				// https://{host}/{path}/remote.php/dav/files/{username}
	/*			if (providerSoftware.equals("Nextcloud")) {
					if( userNameField.getText().isEmpty()) {
						userNameField.setBorder(errorBorder);
						userNameNoteLabel.setText(languageBundle.getString("enter_user_name"));
						userNameNoteLabel.setForeground(Color.RED);
						return;
					} else { 
		// Nextcloud WebDAV base path with user name:				
				String userName = userNameField.getText();
						String webDavBasePath = text.trim() + "remote.php/dav/files/" + userName.trim();
						webDavBaseField.setText(webDavBasePath);
						webDavBaseField.setForeground(Color.BLACK);
						webDavBaseField.setBorder(null);
						webDavNoteLabel.setText(""); /
					}
				} else */ if (providerSoftware.equals("Owncloud")
						|| providerSoftware.equals("Nextcloud")) { // Nextcloud works also with Owncloud path
					// https://{host}/{path}/remote.php/webdav
					String webDavBasePath = text.trim() + "remote.php/webdav";
					webDavBaseField.setText(webDavBasePath);
					webDavBaseField.setForeground(Color.BLACK);
					webDavBaseField.setBorder(null);
					webDavNoteLabel.setText("");
					
				} else if (providerSoftware.equals("Seafile")) {
					// https://{host}/{path}/seafdav
					String webDavBasePath = text.trim() + "seafdav";
					webDavBaseField.setText(webDavBasePath);
					webDavBaseField.setForeground(Color.BLACK);
					webDavBaseField.setBorder(null);
					webDavNoteLabel.setText("");
					
				}
			} else { // serverField empty
				if (webDavBaseField.getText().isEmpty()) {
					serverField.setBorder(errorBorder);
					webDavBaseField.setBorder(errorBorder);
				}
			}
			//this.pack();
		} else if (comp.equals(webDavBaseField)) {
			//TestLog.v(WebDavLogin.class, "webDavBaseField");
			if ( ! webDavBaseField.getText().isEmpty() ) {
				String text = webDavBaseField.getText();
				try {
					@SuppressWarnings("unused")
					URL url = new URL(text);
				} catch (MalformedURLException e1) {
					//TestLog.e(WebDavLogin.class, "Not a valid URL: " + text);
					webDavNoteLabel.setText(languageBundle.getString("enter_valid_web_address"));
					webDavNoteLabel.setForeground(Color.RED);
					webDavBaseField.setBorder(errorBorder);
					return;
				}
				webDavBaseField.setBorder(null);
				webDavBaseField.setForeground(Color.BLACK);
				webDavNoteLabel.setText("");
				// Set the provider name if not present
				if (! text.equals("https://webdav.example.org")) {
					//if (providerField.getText().isEmpty()) {
						String provName = text.substring(text.indexOf("://") + 3);
						if (provName.startsWith("webdav") || provName.startsWith("dav")
								|| provName.startsWith("cloud")
								|| provName.startsWith(userNameField.getText().toLowerCase())){
							if (provName.contains(".")) {
								provName = provName.substring(provName.indexOf(".") + 1);
							}
						}
						if (provName.contains("/")) {
							provName = provName.substring(0, provName.indexOf("/"));
						}
						if (provName.contains(".")) {
							provName = provName.substring(0, provName.indexOf("."));
						}
						if (provName.contains("_")) { // underscore is used in properties
							provName = provName.replace('_', '-');							
						}
						//TestLog.v(WebDavLogin.class, "Set provider name " + provName);
						providerField.setText(provName);
					//}
				}
				if (serverField != null) {
					serverField.setBorder(null);
					serverNoteLabel.setText("");
				}
				//this.pack();
			}
		} else if (comp.equals(userNameField)) {
			
			if ( ! userNameField.getText().isEmpty() ) {
				
				userNameField.setBorder(null);
				userNameNoteLabel.setText("");
			// construct Nextcloud base path...
				if (providerSoftware.equals("Nextcloud") 
						&& serverField != null 
						&& ! serverField.getText().isEmpty() // Nextcloud
						&& serverField.getBorder() == null) { // valid URL
					// construct the WebDAV base path for Nextcloud:
					String userName = userNameField.getText();
					String text = serverField.getText();
					if (text.contains(".") ) {
						// check for index page and remove
						String basePart = text.substring(0, text.lastIndexOf('.'));
						//TestLog.v(WebDavLogin.class, "basePart: " + basePart);
						if (basePart.endsWith("index")  
								|| basePart.endsWith("home") || basePart.endsWith("default")) {
							text = text.substring(0, text.lastIndexOf("/"));
							//TestLog.v(WebDavLogin.class, "text: " + text);
						}
					}
					if (! text.endsWith("/")) {
						text = text + "/";
					}
					String webDavBasePath = text.trim() + "remote.php/dav/files/" + userName.trim();
					webDavBaseField.setText(webDavBasePath);
					webDavBaseField.setForeground(Color.BLACK);
					webDavBaseField.setBorder(null);
					webDavNoteLabel.setText("");
					this.pack();
				}
			} else {
				userNameField.setBorder(errorBorder);
			}
		} else if (comp.equals(passwordField)) {
			if ( passwordField.getPassword().length != 0 ) {
				passwordField.setBorder(null);
			} else {
				passwordField.setBorder(errorBorder);
			}
		} else if (comp.equals(providerField)) {
			if (! providerField.getText().isEmpty() ) {
				providerField.setBorder(null);
			} else {
				providerField.setBorder(errorBorder);
			}
		}
	}


	@Override
	public void insertUpdate(DocumentEvent iue) {
		handleDocumentEvents(iue);
	}


	@Override
	public void removeUpdate(DocumentEvent rue) {
		handleDocumentEvents(rue);
	}


	/**
	 * Handle the placeholder text and color: 
	 * Remove the placeholder text and set color to black
	 */
	@Override
	public void focusGained(FocusEvent fge) {
		// Set or reset the placeholder text
		if (fge.getComponent().equals(serverField)) {
			if (serverField.getText().equals("https//:example.org")) {
				serverField.setText("");
				serverField.setForeground(Color.BLACK);
			}
		} else if (fge.getComponent().equals(webDavBaseField)) {
			if (webDavBaseField.getText().equals("https://webdav.example.org")) {
				webDavBaseField.setText("");
				webDavBaseField.setForeground(Color.BLACK);
			}
		}
	}

	/**
	 * Handle the placeholder text and color: 
	 * Set the placeholder text and set color to gray
	 */
	@Override
	public void focusLost(FocusEvent fle) {
		// show only once, otherwise the placeholder is present for paste action
/*		// Set or reset the placeholder text
		if (fle.getComponent().equals(serverField)) {
		if (serverField.getText().isEmpty()) {
			serverField.setForeground(Color.GRAY);
			serverField.setText("https//:example.org");
        }
		} else if (fle.getComponent().equals(webDavBaseField)) {
			if (webDavBaseField.getText().isEmpty()) {
				webDavBaseField.setText("https://webdav.example.org");
				webDavBaseField.setForeground(Color.GRAY);
			}
		} */
	}



	/**
	 * Inner class for cloud operations
	 */
	private class WebDavLoginWorker extends SwingWorker<String, Void> {

		@Override
		protected String doInBackground() throws Exception { 

			try {
				login.setCursor(CursorManager.getWaitCursor());

				userNameField.setBorder(null);
				userNameNoteLabel.setText("");
				providerField.setBorder(null);
				passwordField.setBorder(null);
				webDavBaseField.setBorder(null);
				if (serverField != null) {
					serverField.setBorder(null);
				}			
				boolean error = false;
				if (providerField.getText().equals("")) {
					providerField.setBorder(errorBorder);
					error = true;
				}
				if (userNameField.getText().equals("")) {
					userNameField.setBorder(errorBorder);
					error = true;
				}
				if (passwordField.getPassword().length == 0) {
					passwordField.setBorder(errorBorder);
					error = true;
				}
				if (webDavBaseField.getText().equals("") ||
						webDavBaseField.getText().equals("https://webdav.example.org")) {
					webDavBaseField.setBorder(errorBorder);
					error = true;
				} else {
					try {
						@SuppressWarnings("unused")
						URL url = new URL(webDavBaseField.getText());
					} catch (MalformedURLException e1) {
						TestLog.e(getClass(), "Invalid WebDAV pasth: " + webDavBaseField.getText());
						webDavBaseField.setBorder(errorBorder);
						webDavNoteLabel.setText(languageBundle.getString("provider_publish_web_dav_path"));
						webDavNoteLabel.setForeground(Color.RED);
						error = true;
					}
				}			
				if (error == false) {
					String providerName = providerField.getText();
					if (providerName.contains("_")) { // underscore is used in properties
						providerName = providerName.replace('_', '-');							
					}
					provider.setProviderName(providerName);
					provider.setWebDavBasePath(webDavBaseField.getText() 
							+ (webDavBaseField.getText().endsWith("/")? "" : "/"));
					String authError = null;

					if (title.equals(PeaProperties.getVmBridge().translate("modify_cloud_provider"))) {
						// do not use opened provider
						authError = provider.updateAuthorization(
								userNameField.getText(), passwordField.getPassword(), null);
					} else {
						// do not open new provider, if provider was already opened
						authError = provider.authorize(
								userNameField.getText(), passwordField.getPassword(), null);
					}
					if (authError == null) {
						if ( ! owner.getName().equals("EncryptedDialog")) { // PeaLockFrame
							((PeaLockFrame)PeaControl.getDialog().getLockFrame()).setMessage(languageBundle.getString("successfully_authorized"), false);
							//Properties props = new Properties();
							String propError = CloudPropertyHandler.setNewProvider(provider.getProviderName(), userNameField.getText(), webDavBaseField.getText());
							if (propError != null) {
								TestLog.e(WebDavLogin.class, "Could not save login values for: " + provider.getProviderName());
								TestLog.ve(WebDavLogin.class, propError, 5);
								return languageBundle.getString("unexpected_error") + "\n" + propError;
							} else {
								((PeaLockFrame)PeaControl.getDialog().getLockFrame())
								.setMessage(languageBundle.getString("provider_added") 
										+ ": " + provider.getProviderName(), false);
							}
							char[] psw = passwordField.getPassword();
					//providerPswMap.put(provider.getProviderName(), psw);
							passwordField.setText("");
							// check if sub set properties exist
							String saltAsHex = Converter.bytes2hex(KeyDerivation.getSalt());
							String algoParam = Converter.bytes2hex(PeaProperties.getAlgoParamString());
							// the extension/marker for the sub set (0 for default set), 
							// or -1 (provider with this salt/algos does not exist), or -2 (unexpected error)
							int ext = CloudPropertyHandler.getSubSetExtension(providerName, 
									saltAsHex, 
									algoParam, null);
							//TestLog.v(getClass(), "ext: " + ext);
							if (ext >= 0) { // provider subset exists: store new password
								CloudPropertyHandler.storeEncryptedPasswordFromProvider(
										CloudPropertyHandler.encryptPassword(new String(psw)), providerName, saltAsHex, algoParam, null);
								Zeroizer.zero(psw);

							} else if (ext == -1) { // sub set does not exist: create subset
								int x = CloudPropertyHandler.createSubSet(providerName, 
										Converter.bytes2hex(KeyDerivation.getSalt()), 
										Converter.bytes2hex(PeaProperties.getAlgoParamString()),
										new String(psw), 
										null);
								Zeroizer.zero(psw);
								if (x == -1) {// failed
									TestLog.e(this.getClass(),  "Could not create sub set for provider " + providerName);
								} else {
									((PeaLockFrame)PeaControl.getDialog().getLockFrame())
									.setMessage(languageBundle.getString("provider_added") 
											+ ": " + provider.getProviderName(), false);
								}
							}

						} else { // PswDialogView open or add

							// keep the password: create a temp key if not already created
							char[] psw = passwordField.getPassword();
							passwordField.setText("");
							byte[] plainPswBytes = Converter.chars2bytes(psw);	
							// get current cipher to reset later
							BlockCipher lastCipher = CipherStuff.getCipherAlgo();
							if (usedCipher == null) {
								// cipher may change:
								usedCipher = CipherStuff.getCipherAlgo();
							} else {
								// set this cipher:
								CipherStuff.setCipherAlgo(usedCipher);
							}
							if (tempKey == null) {
								SecureRandom sc = new SecureRandom();
								tempKey = new byte[CipherStuff.getKeySize()];//.getCipherAlgo().getBlockSize()];
								sc.nextBytes(tempKey);
							}						
							byte[] pswBytes = CipherStuff.processCTR(true, plainPswBytes, tempKey, false);
							Zeroizer.zero(plainPswBytes);
							// Reset:
							CipherStuff.setCipherAlgo(lastCipher);
							TestLog.v(getClass(), "Store manually added password in encrypted form for " + provider);
							providerPswMap.put(provider.getProviderName(), pswBytes);

							// store properties:
							String propError = CloudPropertyHandler.setNewProvider( 
									providerName,
									userNameField.getText(), 
									//null,//CloudPropertyHandler.encryptPassword(psw),
									webDavBaseField.getText());							
							if (propError != null) {
								TestLog.e(WebDavLogin.class, "Could not save login values for: " + provider.getProviderName());
								System.err.println(propError);
								PeaDialog.showMessage(owner, 
										languageBundle.getString("unexpected_error")
										+ "\n" + "couldn't set cloud properties...", 
										languageBundle.getString("error"), 0);
							}

							// last:
							Zeroizer.zero(psw);
						}
						result = 0;
						login.setVisible(false);
						login.dispose();
						return null;
					} else {
						//WebDavLogin.lastAuthSuccess = false;
						TestLog.e(WebDavLogin.class, "Authorization failed...");

						if (authError.contains("timeout")) {
							PeaDialog.showMessage(this, PeaProperties.getVmBridge().translate("timeout"), null, 0);
						//String errorMessage = authError;
							
						} else if (authError.contains("405")) { // WebDAV base path error
							webDavBaseField.setBorder(errorBorder);
						} else if (authError.contains("401")) { // user name or password
							userNameField.setBorder(errorBorder);
							passwordField.setBorder(errorBorder);
						} else {
							webDavBaseField.setBorder(errorBorder);
							userNameField.setBorder(errorBorder);
							passwordField.setBorder(errorBorder);
						}
						((PeaLockFrame)PeaControl.getDialog().getLockFrame())
						.setMessage(authError 
								+ ": " + provider.getProviderName(), true);
						result = 1;
						//PeaDialog.showMessage(owner, provider.getErrorMessages(), languageBundle.getString("authorization_failed"), 0);
						return languageBundle.getString("authorization_failed");
					}
				}
				return null;
			} finally {
				// reset the wait cursor
				login.setCursor(CursorManager.getDefaultCursor());
			}
		}				
		@Override
		protected void done() { 
			//String result = get();
			// reset the wait cursor
			//login.setCursor(CursorManager.getDefaultCursor());
		}
	}

	@Override
	public void windowActivated(WindowEvent arg0) {}

	@Override
	public void windowClosed(WindowEvent e) {}

	@Override
	public void windowClosing(WindowEvent e) {
		TestLog.v(WebDavLogin.class, "closing");
		if (sw != null && sw.getState().equals(SwingWorker.StateValue.STARTED)) {
			sw.cancel(true);
		}
		login.setVisible(false);
		login.dispose();
		result = -1;
	}
	@Override
	public void windowDeactivated(WindowEvent e) {}
	@Override
	public void windowDeiconified(WindowEvent e) {}
	@Override
	public void windowIconified(WindowEvent e) {}
	@Override
	public void windowOpened(WindowEvent e) {}
}