package cologne.eck.all_peas.control;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.SwingUtilities;

import cologne.eck.all_peas.data.AttachmentHandler;
import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.file_pea.FileControl;
import cologne.eck.peafactory.crypto.AlgoParamHandler;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
//import cologne.eck.file_pea.FileControl.EventDispatchThread;
import cologne.eck.tools.Converter;
import cologne.eck.tools.TestLog;


// --file /home/axl/x3 --file /home/axl/x1 --file /home/axl/x2 --passphrase "no password"

// TODOs
// no std:out (silent mode)
// encrypt file with default algorithms
// read path file instead of --file

// Problem: EntropyPool not starting...

public class CommandLineHandler {
	
	private static ArrayList<String> fileList;
	
	private static char[] psw;

	private static final String errorAppendix = "Type \"java -jar YOUR_PEA.jar --help\" to get more information";
	
	private static final String helpText = 
			 "java -jar MY_PEA.jar [-t] [--file file]...\n"
			+ "java -jar MY_PEA.jar [-t] [--file file... [--passphrase passphrase]]\n\n"
			+ "If no passphrase is specified, it is read from standard input.\n\n"
			+ "-t			run in test mode, print infos while running\n"
			+ "--file			file with absolute path\n"
			+ "--passphrase		the passphrase to use for decryption\n"
			+ "			Warning: Use with caution, especially on multi-user systems.\n" 
			+ "			Avoid this option unless you are using the key file property.\n"
			+ "	-h --help	display help and exit\n"
			+ "	--version	output version information and exit\n\n"
			+ "Example: \n"
			+ "	java -jar MY_PEA.jar --file /home/file1 --file /home/file2\n"
			+ "Asks for a passphrase and decrypts and opens file1 and file2";
			
					
//	private static boolean inTestMode = false;

	
	// option passphrase requires file option: 
	// java -jar MY_PEA.jar [-t] [--file file .. [--passphrase passphrase]]
	
	// get passphrase from stdin:
	// java -jar MY_PEA.jar [-t] [--file file]
	
	// java -jar MY_PEA.jar -h --help	display help and exit
	// java -jar MY_PEA.jar -version	output version information and exit
	
	// -r	run in rescue mode (not used yet)
	// -t	run in test mode, print infos while running
	// --file file the file to decrypt and open
	// --passphrase passphrase	enter a passphrase for key derivation


	
	/**
	 * Read and validate the arguments from command line. 
	 * Skip password dialog if the argument contain files. 
	 * 
	 * @param args	arguments from command line
	 * 
	 * @return	false: start password dialog, true: open lock frame directly
	 */
	public static boolean handleArgs(String[] args){
		
		if (args == null) {
			return false;
		}
		int len = args.length;
        for (int i = 0; i < len; i++) {
            TestLog.v(CommandLineHandler.class, "Argument: " + args[i]);
        }
        
		String firstArg = args[0];
		//TestLog.o(getClass(), "firstArg: " + firstArg);
		if ( ! firstArg.startsWith("-")) {
			
			TestLog.e(CommandLineHandler.class, "Invalid parameter " + firstArg + " - must start with a -\n" + errorAppendix);
//			if (inTestMode == false) {
				System.exit(1);
//			}
		}
		if (firstArg.equals("-h") || firstArg.equals("--help")) {
			TestLog.o(CommandLineHandler.class, helpText);
			//if (inTestMode == false) {
			System.exit(0);
			//}
		}
		if (firstArg.equals("--version")) {
			TestLog.o(CommandLineHandler.class, PeaProperties.getVersionInfo());
//			if (inTestMode == false) {
				System.exit(0);
//			} else {
				//return false;
//			}
		}
		
		if (firstArg.equals("-t")) {
			//TestLog.o(getClass(), "firstArg test mode: " + firstArg);
			PeaProperties.setWorkingMode("-t");
			TestLog.v(CommandLineHandler.class, "Run in test mode...");
		} else if (firstArg.equals("-r")) {
				//TestLog.o(getClass(), "firstArg test mode: " + firstArg);
				PeaProperties.setWorkingMode("-r");
				TestLog.v(CommandLineHandler.class, "Run in rescue mode...");				
		} else if (firstArg.startsWith("--file")){
			// OK
		} else {
			TestLog.e(CommandLineHandler.class, "Invalid parameter " + firstArg + "\n" + errorAppendix);
//			if (inTestMode == false) {
			System.exit(1);
			//}
		}
		
		// list the files, get the passphrase:
		fileList = new ArrayList<String>();
		//String passphrase = null;
		for (int i = 0; i < len; i++) {
			String arg = args[i];
			if (arg.equals("--file")) {
				// check for missing file:
				if ( (i + 1) >= len) {
					TestLog.e(CommandLineHandler.class, "Invalid parameter " + args[i] + " - missing specified file." + "\n" + errorAppendix);
//					if (inTestMode == false) {
					System.exit(1);
//					}
				}
				i++;
				fileList.add(args[i]);
			} else if (arg.equals("--passphrase")) {
				// check for missing password:
				if ( (i + 1) >= len) {
					TestLog.e(CommandLineHandler.class, "Missing specified password." + "\n" + errorAppendix);
//					if (inTestMode == false) {
					System.exit(1);
//					}
				}
				i++;
				//passphrase = args[i];
				psw = args[i].toCharArray();
			} else if (arg.equals("-t")) {
				// done
				PeaProperties.setWorkingMode("-t");
			} else if (arg.equals("-r")) {
				// done
				PeaProperties.setWorkingMode("-r");
			} else {
				TestLog.e(CommandLineHandler.class, "Invalid parameter " + args[i]  + "\n" + errorAppendix);
//				if (inTestMode == false) {
				System.exit(1);
//				}
			}
		}
		if (fileList.isEmpty()) {
			return false;
		} else {
			File f = new File(fileList.get(0));
			if (f.exists() == false) {
				TestLog.e(CommandLineHandler.class, "Specified file does not exist: " + fileList.get(0));
				TestLog.e(CommandLineHandler.class, "Program exits...");
				System.exit(1);
			}
			if (f.canRead() == false || f.canWrite() == false) {
				TestLog.e(CommandLineHandler.class, "Missing read/write access to the specified file: " + fileList.get(0));
				TestLog.e(CommandLineHandler.class, "Program exits...");
				System.exit(1);
			}
			byte[][] settings = AttachmentHandler.getSaltAlgosFileIDFromFile(f, false);
			if (settings != null) {

				
				ArrayList<String> validatedList = new ArrayList<String>();
				validatedList.add(fileList.get(0));
				int listLen = fileList.size();
				// Check the other files
				for (int i = 1; i < listLen; i++) {
					File nextFile = new File(fileList.get(i));
					if (nextFile.exists() == false || nextFile.canRead() == false || nextFile.canWrite() == false) {
						TestLog.e(CommandLineHandler.class, "No access: " + fileList.get(i));
						continue;
					}
					byte[][] nextSettings = AttachmentHandler.getSaltAlgosFileIDFromFile(f, false);
					if (nextSettings == null) {
						TestLog.e(CommandLineHandler.class, "Invalid file: " + fileList.get(i));
						continue;
					}
					if (Arrays.equals(settings[2], nextSettings[2]) == true
							&& Arrays.equals(settings[1], nextSettings[1]) == true
							&& Arrays.equals(settings[0], nextSettings[0]) == true) {
						// Add file to validated list:
						validatedList.add(fileList.get(i));
					} else {
						TestLog.e(CommandLineHandler.class, "Cannot open together with first file: " + fileList.get(i));
						continue;
					}					
				}				
				// apply settings from first file:
				KeyDerivation.setSalt(settings[0]);
				//TestLog.o(getClass(), "Salt: " + Converter.bytes2hex(settings[0]));
				//TestLog.o(getClass(), "fileId: " + Converter.bytes2hex(settings[2]));
				String error = AlgoParamHandler.readString(new String(settings[1], AttachmentHandler.getASCIICharset()), false);
				if (error != null) {
					TestLog.e(CommandLineHandler.class, "Unable to set algorithms and parametrers from file " + fileList.get(0));
					TestLog.e(CommandLineHandler.class, "The file " + fileList.get(0) + " seems to be invalid...");
					TestLog.e(CommandLineHandler.class, "Program exits...");
					System.exit(1);
				}
				Attachments.setFileIdentifier(settings[2]);
				
				// set validated list:
				fileList = validatedList;
				
			} else {
				TestLog.e(CommandLineHandler.class, "The file " + fileList.get(0) + " seems to be invalid...");
				TestLog.e(CommandLineHandler.class, "Program exits...");
				System.exit(1);
			}
		}
		
		for (String s : fileList) {
			TestLog.o(CommandLineHandler.class, s);
		}
		//TestLog.o(getClass(), "Passphrase: " + passphrase);
		
		PeaProperties.setCommandLineMode(true);
		
		PeaControl.initializeVariables();				

		SwingUtilities.invokeLater(new EventDispatchThread());
		
		return true;				
	}
	
	//============ inner classes ================
	
	private static class  EventDispatchThread implements Runnable{

		@Override
		public void run() {
			
			// TODO
			PeaControl pswDialog = FileControl.getInstance();		

			//dialogView.setVisible(true);
			PeaControl.setDialog(pswDialog);
/*			if (new File("resources" + File.separator + "pea.properties").exists() == false) { // first time
				HtmlFileViewer hfv = new HtmlFileViewer("first_time_guide", dialogView);
				hfv.requestFocus();
			} */
//			TestLog.v(FileControl.class, "File identifier: " + new String(Attachments.getFileIdentifier()) + " - " + Converter.bytes2hex(Attachments.getFileIdentifier()));
			//model = PeaControl.getEncryptedFileTypePanel().getFileComposer().getFileModel();
			//TestLog.o(getClass(), "files: " + FilePanelHolder.getFileNumber());
/*			if (FilePanelHolder.getFileNumber() == 0) {
			//if (((FileTypePanel)JREProperties.getTypePanel()).getFileComposer().getFileModel().getFileNumber() == 0 ) {
				PswDialogView.setMessage(PeaProperties.getVmBridge().translate("open_or_initialize"), true, false);
			} else {
				
				FilePanelHolder.updateWindow();
				//dialogView.pack();
			} */

			PeaControl.getDialog().startExecution(false);
		}		
	}

	public static String[] getFileNames() {
		return Converter.arrayListToArray(fileList);
	}

	static char[] getPassword() {
		if (psw == null) {

			//psw = System.console().readLine().toCharArray(); 

			if (System.console() != null) {
				TestLog.o(CommandLineHandler.class, "Enter the password:");
				psw = System.console().readPassword();
			} else {
				TestLog.o(CommandLineHandler.class, "!!! "
						+ "Warning: Missing console...");
				TestLog.o(CommandLineHandler.class, "Enter the password:");
				//Enter data using BufferReader 
				BufferedReader reader =  
						new BufferedReader(new InputStreamReader(System.in)); 



				// Reading data using readLine 
				try {
					psw = reader.readLine().toCharArray();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} finally {
					try {
						reader.close();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
		}
		return psw;
	}

	/**
	 * @return the inTestMode
	 */
/*	public static boolean isInTestMode() {
		return inTestMode;
	}*/

	/**
	 * @param inTestMode the inTestMode to set
	 */
/*	public static void setInTestMode(boolean inTestMode) {
		CommandLineHandler.inTestMode = inTestMode;
	}*/
}
