package cologne.eck.image_pea;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */



/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License
 * along with this library.
 */

/**
 * Dialog to get the password and start the image pea.
 */

/*
 * Password dialog to decrypt image.
 * Image is displayed in LockFrameImage.
 *
 * Password can not be changed and content (image) can not be modified.
 *
 * cipher text is stored in external file or inside jar
 *
 */


import java.awt.Point;
import java.awt.Window;
import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Properties;
//import java.util.zip.CRC32;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import cologne.eck.all_peas.files.FilePanelHolder;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.files.InvalidPanel;
import cologne.eck.all_peas.files.CloudFilePanel;
import cologne.eck.all_peas.files.FileComposer;
import cologne.eck.all_peas.files.FileModel;
import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.CloudPropertyHandler;
import cologne.eck.all_peas.control.CommandLineHandler;
//import cologne.eck.all_peas.files.FilePanelHolder;
//import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.FileRecipient;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.AttachmentHandler;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.HtmlFileViewer;
import cologne.eck.all_peas.gui.KeyFileDialogs;
import cologne.eck.all_peas.gui.MultiFileSelectionDialog;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PswDialogView;
//import cologne.eck.all_peas.gui.Toaster;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.AlgoParamHandler;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.RandomStuff;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.tools.Converter;
import cologne.eck.tools.EntropyPool;
import cologne.eck.tools.FileTools;
import cologne.eck.tools.PropertyHandler;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.Zeroizer;


final class ImageControl extends PeaControl {

	private static int imageWidth = 800;
	private static int imageHeight = 600;
//	private static boolean resize = true;

	private static PswDialogView dialogView;

	private static ImageControl pswDialog;
	private static LockFrameImage lockFrame = null;
	private static String peaPropFileName = "image.properties";
	private static final String[] supportedExtensions = {"png", "gif", "bmp", "jepg", "jpg"}; 
	
	private static final String PEA_NAME = "Image-Lock-PEA";
	private static final String VERSION = "1.3 (PEA version 2)";
	private static final String YEAR_OF_PUBLICATION = "2021";
	private static final String WEBSITE = "/html/image_pea.html";
	private static final String SOURCE_LINK = "http://eck.cologne/peafactory/src/downloads/imagePEA/imagePEA-1.3-src.zip";

	// to show errors and failed files only once: collect errors
	private StringBuilder errorsToShow = new StringBuilder();

	private ImageControl() {

		pswDialog = this;
		setDialog(pswDialog);
		dialogView = PswDialogView.getInstance();
		JREProperties.setMainWindow(dialogView);
		PeaControl.setPlainTextMode(false);
	}

	private final static ImageControl getInstance() {
		ImageControl dialog = null;
		if (pswDialog == null) {
			dialog = new ImageControl();
		} else {
			// return null
		}
		return dialog;
	}

	public static void main(String[] args) {
		
		PeaProperties.setVersionInfo(PEA_NAME + " " + VERSION);
		PeaProperties.setFileType("image");	
		PeaControl.setPeaPropFileName(peaPropFileName);
		new PropertyHandler().readAndSetAllPropValues();
		if(args.length > 0) {
			//PeaProperties.setWorkingMode(args[0]);
			boolean runCommands = CommandLineHandler.handleArgs(args);
			TestLog.v(ImageControl.class, "Run in command line mode: " + runCommands);
		}		
		if (PeaProperties.isCommandLineMode() == false) {
			PeaControl.initializeVariables();		
			SwingUtilities.invokeLater(new EventDispatchThread());		
		} else {
			TestLog.v(ImageControl.class, "Command line mode...");
		}
	}

	@Override
	public final LockFrameImage getLockFrame() {
		return lockFrame;
	}

	//-------------------HELPER-FUNCTIONS--------------

	@Override
	public void clearSecretValues() {
		// is done in LockFrameImage to handle initialization
		//LockFrameImage.clearImageBytes();
	}
	private void publish(String message, boolean isError) {
		PswDialogView.setMessage(message, isError, false);
	}
	/**
	 * Reset cursor, enable ok button and optionally
	 * update the view of the password dialog.
	 * This is called in the done method only.
	 * 
	 * @param returnToDialog	true, if the program should return to the 
	 * 							password dialog and check boxes were added or removed,
	 * 							eventually set salt and key are to be cleared
	 * 							(an error occurred)
	 */
	private void cleanUpOnReturn(boolean returnToDialog) {
		// show collected error messages:
		if (errorsToShow != null && errorsToShow.length() > 0) {
			PeaDialog.showMessage(dialogView, errorsToShow, PeaProperties.getVmBridge().translate("error"), 0);
		}
		// reset errors:
		errorsToShow = new StringBuilder();
		dialogView.setCursor(CursorManager.getDefaultCursor());
		if (returnToDialog == true) { // failed
			PswDialogView.enableOkButton();
			KeyFileDialogs.resetProbablyNewDefaultKeyFileName();
			CipherStuff.getSessionKeyCrypt().clearKeys();
			// maybe files were removed...
			FilePanelHolder.updateFileView();
			PswDialogView.clearPassword();
			dialogView.pack();
			JREProperties.setMainWindow(dialogView);
			PeaControl.setPlainTextMode(true);
		} else {
			JREProperties.setMainWindow(lockFrame);
			PeaControl.setPlainTextMode(false);
		}
	}

	@Override
	public void startDecryption() {

		//ArrayList <String> fileNames = null;
		String[] fileNames = null;
		publish(PeaProperties.getVmBridge().translate("decryption") + "...", false);

		if (PswDialogView.isInitializing() == false) {
			
			if ( CipherStuff.getSessionKeyCrypt().isKeySet() == true) {
				TestLog.e(getClass(), "Existing key before KDF...");
				cleanUpOnReturn(true);
			}
			if (KeyDerivation.getSalt() == null) {
				TestLog.e(getClass(), "Missing salt...");
				cleanUpOnReturn(true);
			}

			// derive the key
			byte[] keyMaterial;
			// KDF: derive the key from password and salt: 
			publish(PeaProperties.getVmBridge().translate("key_derivation") + "... ", false);
			keyMaterial = getKeyMaterial();
			if (keyMaterial == null) {
				TestLog.e(ImageControl.class, "key derivation failed");
				if (PeaProperties.isTestMode()) {
					new UnexpectedValueException("keyMaterial", "byte[]", "is null").printDescription();
				}
				errorsToShow.append("\n" + PeaProperties.getVmBridge().translate("unexpected_error") + " (kdf)");					
				//PeaDialog.showMessage(dialogView, errorsToShow, PeaProperties.getOsBridge().translate("error"), 0);
				publish(PeaProperties.getVmBridge().translate("unexpected_error") + " (kdf)", true);
				cleanUpOnReturn(true);
				return;
			} else {
				CipherStuff.getSessionKeyCrypt().storeKey(keyMaterial);
			}

			FileRecipient frw = new FileRecipient();
			fileNames = frw.getFileNames();

			// From FileRecetionWorker: collect to show later with other errors
			if (frw.getCollectedErrorsToShow() != null && frw.getCollectedErrorsToShow().length() > 0) {
				errorsToShow.append( frw.getCollectedErrorsToShow());
			}
			if (fileNames == null) {
				publish(PeaProperties.getVmBridge().translate("no_valid_file_found"), true);
				cleanUpOnReturn(true);
				return;
			}
			
			HashMap<String, HashSet<String>> removeMap = frw.getProviderAndFilesToRemove();
			if (removeMap != null && removeMap.size() > 0) {
				Iterator<HashMap.Entry<String, HashSet<String>>> it = removeMap.entrySet().iterator();
				while (it.hasNext()) {
					HashMap.Entry<String, HashSet<String>> pair = it.next();
					HashSet<String> set = pair.getValue();
					for (String s : set) {
						//TestLog.o(getClass(), s);
						this.getEncryptedFileTypePanel().getCloudFilePanel().getCloudControl().removeInitialFileOfProvider(pair.getKey(), s);
						// show as invalid file
						InvalidPanel.addInvalidFileCheckBox( pair.getKey() + "-" + s, PeaProperties.getVmBridge().translate("error") 
								+ " - " + (String) pair.getKey() + FileModel.getInvalidMarker());
					}
					//TestLog.o(getClass(), pair.getKey() + " = " + pair.getValue());
					it.remove(); // avoids a ConcurrentModificationException
				}
			}

			// set first file as image to show:
			int firstFileIndex = 0;
			int len = fileNames.length;
			// get first file in fileNames:
			for (int i = 0; i < len; i++) {
				if(new File(fileNames[i]).isFile()) {
					firstFileIndex = i;
					break;
				}
			}
			byte[] cipherBytes =  null;
			if (fileNames[firstFileIndex].equals(PeaProperties.getVmBridge().translate("image_inside_jar_archive"))){
				cipherBytes = ReadResources.getResourceFromJAR("resources/text.lock");
			} else {
				cipherBytes = ReadResources.readExternFile(fileNames[firstFileIndex]);
			}
			if (cipherBytes == null) {
				new UnexpectedValueException("cipherBytes", "byte[]", "is null for: " + fileNames[firstFileIndex]).printDescription();
			}
	    	// decrypt in RAM:
	    	byte[] imageBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );
			if (imageBytes == null) {
				new UnexpectedValueException("imageBytes", "byte[]", "is null for: " + fileNames[firstFileIndex]).printDescription();
			}
	    	LockFrameImage.setFileList(new ArrayList<String>(Arrays.asList(fileNames)));
	    	PeaControl.setAvailableFileNames(new ArrayList<String>(Arrays.asList(fileNames)));

	    	LockFrameImage.setImageBytes(imageBytes);
    		if (lockFrame != null) {    			
    			setLockFrame(LockFrameImage.getInstance(PswDialogView.getView().getLocation()));
    			//lockFrame = LockFrameImage.getInstance(PswDialogView.getView().getLocation());
    			lockFrame.setMessage(fileNames[firstFileIndex]);
    		}

		} else { // initializing...
			
		/*	if ( (PeaControl.getAvailableFileNames() == null || PeaControl.getAvailableFileNames().isEmpty())
					&& 
					(LockFrameImage.getFileList() == null || LockFrameImage.getFileList().isEmpty()) ) {
				// no file to initialize was found
				TestLog.o(getClass(), "No initialized files found...");
				cleanUpOnReturn(true);
				publish(PeaProperties.getVmBridge().translate("no_valid_file_found"), true);
				PswDialogView.setInitializing(false);
				return;
			}*/
			
			publish(PeaProperties.getVmBridge().translate("init_new_file") + "... ", false);
			// set new salt to attach to file 
			byte[] lastSalt = null;//PeaProperties.getLastSalt();

			// take salt from last selected FileTypePanel if there is one
			if (KeyDerivation.getSalt() != null && FilePanelHolder.getFileNumber() > 0) { 
				lastSalt = KeyDerivation.getSalt();
			} else {
				// get last salt from properties:
				byte[] propSalt = PeaProperties.getLastSalt();
				if (propSalt != null) {
					lastSalt = propSalt;
				}
			}			
			
			if (lastSalt != null) {
				// ask
				int result = PeaDialog.showQuestion(
						//FileControl.dialogView, 
						PswDialogView.getView(),
						//		PeaProperties.getOsBridge().translate("add_new_files_when_decrypted") + "\n" +
						PeaProperties.getVmBridge().translate("open_files_together"), 
						PeaProperties.getVmBridge().translate("init_new_file"),
						0, 1);
				if (result == 0){ // yes  ok
					KeyDerivation.setSalt(lastSalt);
				} else {
					KeyDerivation.setSalt(
							new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize()));
				}
			} else {
				// FilePanelHolder should created a salt... so this should not happen
				if (KeyDerivation.getSalt() == null) {
					KeyDerivation.setSalt(
						new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize()));
				}
			}
			// key was derived in initializeNewFiles()
			// derive key from password dialog:
			if (CipherStuff.getSessionKeyCrypt() == null 
					||  CipherStuff.getSessionKeyCrypt().isKeySet() == false) {
				byte[] keyMaterial = getKeyMaterial();
				// store the key to use later:
				CipherStuff.getSessionKeyCrypt().storeKey(keyMaterial);		
			}
			
			// salt and nonce are set, key derived, file was prepared, 
			// model was set in initializeNewFile
			fileNames = Converter.arrayListToArray(PeaControl.getAvailableFileNames());
			
			if (JREProperties.getTypePanel() == null) {
				// set new FileTypePanel, CloudFilePanel...
				FileComposer newFc = new FileComposer(new FileModel(false, PeaProperties.getAlgoParamString(), KeyDerivation.getSalt()));
				FileTypePanel ftp = new FileTypePanel(100,100, true, false, newFc, JREProperties.getMainWindow());
				// set new panel:
				JREProperties.setTypePanel(ftp);
				CloudFilePanel newCfp = new CloudFilePanel(JREProperties.getMainWindow(), ftp, Converter.bytes2hex(KeyDerivation.getSalt()), 
						Converter.bytes2hex(PeaProperties.getAlgoParamString()));
				// set CloudFilePanel
				ftp.setCloudFilePanel(newCfp);
			}

			TestLog.o(getClass(), "Initialization...");		
		}
		// lockFrame:
		// change background color
		PswDialogView.setUI(false);
		
		if (lockFrame == null) {
			new PropertyHandler().setSpecificPropValue();
			int[] p = JREProperties.getFrameLocation();
			try {
				Point pV = PswDialogView.getView().getLocation(); 
				lockFrame = LockFrameImage.getInstance(pV);
				//setLockFrame(LockFrameImage.getInstance(pV));
			} catch (Exception e) {
				System.err.println(e.getMessage() + ", " + e.toString());
				lockFrame = LockFrameImage.getInstance(new Point(p[0], p[1]));
				//setLockFrame(LockFrameImage.getInstance(new Point(p[0], p[1])));
			}
			//lockFrame = LockFrameImage.getInstance(p);
		}
		
		// set the CloudFileModel with encrypted files
		// add cloud files to model, remove null files
		ArrayList<String> fileNamesList = new ArrayList<String>();
		for (String validFileName : fileNames) {
			if (validFileName != null) {
				fileNamesList.add(validFileName);
				if (validFileName.contains(CloudControl.getTmpDir())) {
					//TestLog.v(EditorControl.class, "Add to CloudFileModel: " + validFileName, 5);
					this.getEncryptedFileTypePanel().getCloudFilePanel().getCloudControl().addUsedFileNameToProvider(
							CloudControl.getProviderNameFromTmpFile(validFileName), 
							CloudControl.getFileNameFromTmpFile(validFileName));
				}
			} else {
				TestLog.ve(ImageControl.class, "valid file name is null", 5);
			}
		}
		fileNames = Converter.arrayListToArray(fileNamesList);
		
		// collect the duplicates to set associated file names of duplicates later
		//ArrayList<HashSet<String>> listOfDuplicateSets = new ArrayList<HashSet<String>>();
		
		//checkDuplicateContent(fileNames, //validFileWithNotesMap, 
		//		listOfDuplicateSets);

		PswDialogView.getView().setVisible(false);

		lockFrame.setVisible(true);
		setLockFrame(lockFrame);
		JREProperties.setMainWindow(lockFrame);
		PeaControl.setPlainTextMode(true);

		
/*		ArrayList<HashSet<String>> setList = ImageModel.getSetList();
		for (HashSet<String> set : setList) {
			for (String s : set) {
				if (lockFrame.decryptAndLoadFileToView(s) == null) {// no error
					break;
				} else {
					// try next file name
				}
			}
		}*/

		EntropyPool.getInstance().stopCollection();
		
		if (errorsToShow != null && errorsToShow.length() > 0) {
			// show all collected errors
			PeaDialog.showMessage(JREProperties.getMainWindow(), errorsToShow, PeaProperties.getVmBridge().translate("error"), 0);
			// reset errors
			errorsToShow = new StringBuilder();
		}
		// Add successfully added files to cloud.properties
		if (this.getEncryptedFileTypePanel() != null) { //  null in initialization
			CloudFilePanel cfp = this.getEncryptedFileTypePanel().getCloudFilePanel();
			if (cfp != null) {
				String[] providers = cfp.getCloudControl().getUsedProviders();
				if (providers != null && providers.length > 0) {
					int providersLen = providers.length;
					for (int i = 0; i < providersLen; i++) {

						String[] usedFiles = cfp.getCloudControl().getUsedFileNamesOfProvider(providers[i]);
						String error = CloudPropertyHandler.addFileNamesToProperties(providers[i], usedFiles,
								Converter.bytes2hex(KeyDerivation.getSalt()), Converter.bytes2hex(PeaProperties.getAlgoParamString()));
						if (error != null) {
							TestLog.ve(this.getClass(), error);
						}

					}
				}
			}
		}
		PswDialogView.getView().setVisible(false);		
		cleanUpOnReturn(false);
		if (lockFrame != null) {
			setLockFrame(lockFrame);
		} else {
			TestLog.e(getClass(), "lockFrame is null");
			TestLog.ve(getClass(), "lockFrame is null",  5);
		}
/*		StringBuilder toastBuilder = new StringBuilder();
		for (HashSet<String> set : listOfDuplicateSets) {
			if (set.size() > 1) {
				toastBuilder.append(PeaProperties.getVmBridge().translate("synchronized" ) + ": \n");
				for (String s : set) {
					toastBuilder.append("   " + s + "\n");
				}
				toastBuilder.append("\n");
			}
		}*/
		// there is no synchronization: for later use (modify images...)
//		if (toastBuilder.length() > 1) { // show synchronized files
//			Toaster.toast(JREProperties.getMainWindow(), new String(toastBuilder), 3);
//		}
	}
	private static synchronized void setLockFrame(LockFrameImage newLockFrame) {
		lockFrame = newLockFrame;
	}

	// not used here:
	@Override
	public void preComputeInThread() {
		checkUpdates();
	}
	@Override
	public String[] getSelectedFileNames() {
		return FilePanelHolder.getSelectedPanel().getValidLocalFileNames(true, true);
	}
	
	/**
	 * Check the list of file names for duplicates:
	 * check 1. file size, 2. file name and  3. checksum of encrypted content.
	 * Others than Calendar Lock PEA, there is no decryption here. 
	 * If file names are equal, but not size/checksum: Ask which one to open
	 * 
	 * @param fileNamesToCheck				the file names to check
	 * @param decryptedFileWithCalendarsMap		map of successfully decrypted files (whether or not duplicate and contain calendars)  
	 * 								with RTF texts in that file
	 * @param listOfDuplicateSets	a list of sets of duplicate file names (each set contains file names that are duplicate)
	 */
/*	private void checkDuplicateContent(	String[] fileNamesToCheck, 
			//HashMap<String, HashSet<PCalendar>> decryptedFileWithCalendarsMap, 
			ArrayList<HashSet<String>> listOfDuplicateSets) {
		
		// iterate over file names: 
		int len = fileNamesToCheck.length;
		
		// Generate a map of files to be opened
		// key: file name without path
		// value: 	0: index of file in fileNamesToCheck
		//			1: file size of files
		//			2: checksum of encrypted files
		HashMap<String, ArrayList<long[]>> fileNameIndexMap = new HashMap<String, ArrayList<long[]>>();

		CRC32 crc = new CRC32();
		//long newestLastMod = 0;

		for (int i = 0; i < len; i++) {

			if ( fileNamesToCheck[i] != null) {
				File file = new File( fileNamesToCheck[i] );
				//	TestLog.v(this.getClass(), "*** check: "+fileNamesToCheck[i] + " size: " + file.length());
				if ( file.isFile()) { // checks also existence

					long fileSize = file.length(); // get file length
					// access, pswIdentifier... already checked in FileRecipient
					crc.update(ReadResources.readExternFile(fileNamesToCheck[i]));
					long  checksum1 = crc.getValue(); // get crc checksum
					crc.reset();
					// 1. check file name
					String nameOfFile = file.getName();
					if (fileNameIndexMap.containsKey(nameOfFile)) {
						// values of first file in list (values  of size and checksum are equal)
						long[] values = fileNameIndexMap.get(nameOfFile).get(0);
						if (fileSize == values[1]) {

							if (checksum1 == values[2]) {
								// add new file to key:
								long[] newValues = {i, fileSize, checksum1};
								fileNameIndexMap.get(nameOfFile).add(newValues);
							} else {
								// ask user, which file should be opened

								// create new key:
								long[] newValues = {i, fileSize, checksum1};
								ArrayList<long[]> list = new ArrayList<long[]>();
								list.add(newValues);
								// put a new entry into the map
								fileNameIndexMap.put(nameOfFile, list);

							}

						} else {
							// ask user, which file should be opened

							// create new key:
							long[] newValues = {i, fileSize, checksum1};
							ArrayList<long[]> list = new ArrayList<long[]>();
							list.add(newValues);
							// put a new entry into the map
							fileNameIndexMap.put(nameOfFile, list);

						}
					} else {
						// create new key:
						long[] newValues = {i, fileSize, checksum1};
						ArrayList<long[]> list = new ArrayList<long[]>();
						list.add(newValues);
						// put a new entry into the map
						fileNameIndexMap.put(nameOfFile, list);
					}
				} else { //  file exists
					new UnexpectedValueException("File", "file", "does not exist: " + fileNamesToCheck[i]).printDescription();
				}
			} else {
				TestLog.ve(getClass(), "file name is null", 5);
			}
		} // for loop fileNamesToCheck
		
		// add files to model and handle cloud files
		for (HashMap.Entry<String, ArrayList<long[]>> entry : fileNameIndexMap.entrySet()) {
			
//			  String fileNameInMap = entry.getKey();
			  ArrayList<long[]> valueList = entry.getValue();

			  // collect all equal file names with path
			  HashSet<String> set = new HashSet<String>();

			  for (long[] val : valueList) {
				  String fileNameWithPath = fileNamesToCheck[ (int)val[0] ];
				  set.add(fileNameWithPath);
					// add provider to CloudControl
					if (fileNameWithPath.contains(CloudControl.getTmpDir())) {
						String providerName = CloudControl.getProviderNameFromTmpFile(fileNameWithPath);
						String propFileName = CloudControl.getFileNameFromTmpFile(fileNameWithPath);
			//			CloudPropertyHandler.setSingleProperty(providerName + "_files", propFileName);
						// TODO test
						String[] filesToAdd = {propFileName};
						String error = CloudPropertyHandler.addFileNamesToProperties(providerName, filesToAdd,
								Converter.bytes2hex(KeyDerivation.getSalt()), Converter.bytes2hex(PeaProperties.getAlgoParamString()));
						if (error != null) {
							TestLog.e(ImageControl.class, error);
							TestLog.ve(ImageControl.class, "Adding file names to properties failed: " + error,5);
						}
					}
			  }
			  listOfDuplicateSets.add(set);
			  ImageModel.addFileNameSet(set);
		}
	}*/


	protected static final void setImageWidth(int newWidth) {
		imageWidth = newWidth;
	}
	protected static final void setImageHeight(int newHeight) {
		imageHeight = newHeight;
	}
/*	protected static final void setChangeSize(boolean newResize) {
		resize = newResize;
	}*/
	protected static final int getImageWidth(){
		return imageWidth;
	}
	protected static final int getImageHeight(){
		return imageHeight;
	}

	@Override
	public String initializeNewFiles() {// 
		// is called from LockFrameImage add new files: add plain text files (current algos)
		// and initialize in PswDialogView:  add plain text files (default algos)

		StringBuilder errorBuilder = new StringBuilder();
		
		if (JREProperties.getMainWindow() instanceof PswDialogView ) {
			// get default values from properties
			String algoParamString = PeaProperties.getDefaultAlgoParamString();//new String(PeaProperties.getAlgoParamString(), AttachmentHandler.getASCIICharset());
			TestLog.v(ImageControl.class, "Default algorithm-parameter string: " + algoParamString);
			// set to use these settings
			AlgoParamHandler.readString(algoParamString, false);
			// set algo parameter string to use
			PeaProperties.setAlgoParamString(algoParamString.getBytes(AttachmentHandler.getASCIICharset()));//AlgoParamHandler.writeString().getBytes(AttachmentHandler.getASCIICharset()));		
		} else {
			// use current algos 
		}

		Window win = JREProperties.getMainWindow();

		FileFilter imageFilter = new FileNameExtensionFilter(
				"Image files", getSupportedExtensions());//ImageIO.getReaderFileSuffixes());

		// show a chooser which shows only image files and directories
		// and which allow to select folders:
		PeaFileChooser chooser = new PeaFileChooser(win);
		chooser.setFileFilter(imageFilter);
		chooser.setMultiSelectionEnabled(true);
		chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		chooser.customizeFileChooser(
				PeaProperties.getVmBridge().translate("select_images_to_encrypt"),
				PeaProperties.getVmBridge().translate("select_single_image_or_folder"),
				false);// no hidden files option

		int returnVal = chooser.showOpenDialog(win);

		if(returnVal == JFileChooser.APPROVE_OPTION) {	// ok

			boolean containsValidImageFile = false;
			ArrayList<String> checkedList = new ArrayList <String>();

			try {
				File[] selectedFiles = chooser.getSelectedFiles();//.getSelectedFile();

				if (selectedFiles == null) {
					TestLog.ve(getClass(), "No file to initialize selected...");
					PswDialogView.setInitializing(false);
					return PeaProperties.getVmBridge().translate("no_valid_file_found");
				}
				// set wait cursor for image checking...
				win.setCursor(CursorManager.getWaitCursor());
				// check if at least one image is in the folder,
				// check if images are valid and add valid files to list
				ArrayList<File> containingFilesList = new ArrayList<File>();;
				for (File selectedFile : selectedFiles) {
					if (selectedFile.isFile()){
						containingFilesList.add(selectedFile);
					} else if (selectedFile.isDirectory()){ // directory was selected
						// show warning message for large size or number of files:
						FileModel model = FilePanelHolder.getSelectedModel();
						//((FileTypePanel) JREProperties.getTypePanel()).getFileComposer().getFileModel();
						boolean printErrors = false; 
						if (PeaProperties.isTestMode()){
							printErrors = false;
						}
						if (model.checkNumberAndSizeOfFolder(selectedFile, this, true, printErrors) == false){
							TestLog.ve(getClass(), "Initialization cancelled...");
							publish(PeaProperties.getVmBridge().translate("no_valid_file_found"), true);
							PswDialogView.setInitializing(false);
							return PeaProperties.getVmBridge().translate("no_valid_file_found");
						}
						//containingFiles = FileTools.getAllFilesOfFolder(selectedFile, true);
						// reset wait cursor
						win.setCursor(CursorManager.getWaitCursor());
						// get all file including from sub-directories, but no directories
						File[] toAdd = FileTools.getAllFilesOfFolder(selectedFile, true);
						containingFilesList.addAll(Arrays.asList(toAdd));//selectedFile.listFiles();
						//PswDialogView.setMessage("Program is working - this may take some time...");
					}
				}
				if (containingFilesList.isEmpty() == true) {
					JOptionPane.showMessageDialog(
							win,
							PeaProperties.getVmBridge().translate("no_image_found"),
							PeaProperties.getVmBridge().translate("error"),
							JOptionPane.ERROR_MESSAGE);
					TestLog.e(getClass(), "No file to initialize found...");
					TestLog.ve(getClass(), "No file to initialize found...", 5);
					// reset the cursor:
					win.setCursor(CursorManager.getDefaultCursor());
					PswDialogView.setInitializing(false);
					publish(PeaProperties.getVmBridge().translate("no_valid_file_found"), true);
					return PeaProperties.getVmBridge().translate("no_valid_file_found");
				}
				File[] containingFiles = containingFilesList.toArray(new File[0]);


				int len = containingFiles.length;
				if (len > 3) {
					PswDialogView.setMessage(PeaProperties.getVmBridge().translate("work_in_progress"), false, false);
				}
				if (len == 1) {
					// for single file, check only if image, not MIME type, not extension:
					if (ImageHelper.checkImage(containingFiles[0], false, false, true) == true) {
						TestLog.v(getClass(), "Check image format: " + containingFiles[0].getAbsolutePath());
						// add the file name to the list
						checkedList.add(containingFiles[0].getAbsolutePath());
						containsValidImageFile = true;
					} else {
						TestLog.e(getClass(), "Invalid format: " + containingFiles[0].getAbsolutePath());
						errorBuilder.append(PeaProperties.getVmBridge().translate("invalid_image_file") + containingFiles[0].getAbsolutePath() + "\n");
					}
				} else {

					for (int i = 0; i < len; i++) {
						// check also extension, but not MIME type
						if (ImageHelper.checkImage(containingFiles[i], true, false, true) == true) {
							TestLog.v(getClass(), "Check image format: " + containingFiles[0].getAbsolutePath());
							// add the file name to the list
							checkedList.add(containingFiles[i].getAbsolutePath());
							containsValidImageFile = true;
						} else {
							TestLog.e(getClass(), "Invalid format: " + containingFiles[i].getAbsolutePath());
							errorBuilder.append(PeaProperties.getVmBridge().translate("invalid_image_file")  + containingFiles[i].getAbsolutePath() + "\n");
						}
					}
				}
			} finally {
				// reset the cursor: checking image files
				win.setCursor(CursorManager.getDefaultCursor());
			}
			if (containsValidImageFile == false || checkedList.isEmpty()) {
				JOptionPane.showMessageDialog(
						win,
						PeaProperties.getVmBridge().translate("no_image_found"),
						PeaProperties.getVmBridge().translate("error"),
						JOptionPane.ERROR_MESSAGE);
				// reset the cursor:
				//win.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				TestLog.ve(getClass(), "No image file found", 5);
				PswDialogView.setInitializing(false);
				publish(PeaProperties.getVmBridge().translate("no_valid_file_found"), true);
				return PeaProperties.getVmBridge().translate("no_valid_file_found");
			} else {
				// sort list:
				Collections.sort(checkedList);

				// show all image files as check boxes and ask to encrypt
				MultiFileSelectionDialog ifd = new MultiFileSelectionDialog(win,
						win.getLocationOnScreen(),
						PeaProperties.getVmBridge().translate("choose_file_to_encrypt"),
						checkedList);
				// get selected image files:
				checkedList = ifd.getSelectedFiles();
				TestLog.v(getClass(), "Number of selected files to initialize: " + ( (checkedList == null) ? "0" : checkedList.size()) );
				// sort the list
				if (checkedList == null || checkedList.isEmpty()) { // cancelled
					TestLog.ve(getClass(), "initializing cancelled..");
					ifd.setVisible(false);
					ifd.dispose();
					PswDialogView.setInitializing(false);
					return PeaProperties.getVmBridge().translate("no_valid_file_found");
				}
				ifd.setVisible(false);
				ifd.dispose();

				try {
					// set wait cursor for encryption
					win.setCursor(CursorManager.getWaitCursor());
					
					byte[] keyMat = null;

					if (win instanceof PswDialogView) {
						// derive the key from password
						keyMat = getKeyMaterial();
						// store session key:
						CipherStuff.getSessionKeyCrypt().storeKey(keyMat);
					} else {//if (win instanceof PeaLockFrame) {
						keyMat = CipherStuff.getSessionKeyCrypt().getKey();
					}
					if (keyMat == null) {
						PswDialogView.setInitializing(false);
						TestLog.ve(getClass(), "key derivation failed...", 5);
						publish(PeaProperties.getVmBridge().translate("unexpected_error"), true);
						win.setCursor(CursorManager.getDefaultCursor());
						return PeaProperties.getVmBridge().translate("unexpected_error");
					}

					int keyLen = keyMat.length;

					ArrayList<String> encryptedList = new ArrayList<String>();

					for (int i = 0; i < checkedList.size(); i++) {
						//TestLog.o(getClass(), "encrypt: " + list.get(i));
						// the key will be overwritten
						byte[] key = Arrays.copyOf(CipherStuff.getSessionKeyCrypt().getKey(), keyLen);
						// TODO same content: always same IV
						boolean success = ContentUtilities.saveContent(key,
								ReadResources.readExternFile(checkedList.get(i)), null, 
								checkedList.get(i), true);
						if (success == false) {
							TestLog.e(getClass(), "Failed to encrypt " + checkedList.get(i));
							errorBuilder.append(PeaProperties.getVmBridge().translate("encryption_failed")  + checkedList.get(i) + "\n");
							// set the wait cursor again: modal error dialog may reset cursor
							win.setCursor(CursorManager.getWaitCursor());

						} else {
							TestLog.v(getClass(), "Encrypted: " + checkedList.get(i));
							if (checkedList.get(i) != null) {
								encryptedList.add(checkedList.get(i));
							} else {
								TestLog.ve(getClass(), "Null file found ", 5);
							}
						}
					}
					Zeroizer.zero(keyMat);

					if (encryptedList.isEmpty()) {
						JOptionPane.showMessageDialog(
								win,
								PeaProperties.getVmBridge().translate("no_image_found"),
								PeaProperties.getVmBridge().translate("error"),
								JOptionPane.ERROR_MESSAGE);
						TestLog.ve(getClass(), "No encrypted file left ", 5);
						// reset the cursor:
						win.setCursor(CursorManager.getDefaultCursor());
						PswDialogView.setInitializing(false);
						publish(PeaProperties.getVmBridge().translate("no_valid_file_found"), true);
						return PeaProperties.getVmBridge().translate("no_valid_file_found");
					}

					// add to available files to set path file
					// forget previously shown file names...
					if (lockFrame != null) {
						// add new file in LockFrame
						//if(list!=null) { for(int i=0;i<list.size();i++) TestLog.o(getClass(), i + ": " + list.get(i));}
						TestLog.v(getClass(), "Adding " + encryptedList.size() + " files to available file list");
						PeaControl.getAvailableFileNames().addAll(encryptedList);
						Collections.sort(PeaControl.getAvailableFileNames());

						int encryptedListSize = encryptedList.size();
						for (int i = 0; i < encryptedListSize; i++) {
							LockFrameImage.addFileToList(encryptedList.get(i));
						}

					} else {
						// initialize in password dialog
						TestLog.v(getClass(), "Initializing list of " + encryptedList.size() + " available image files");
						PeaControl.setAvailableFileNames(encryptedList);
						LockFrameImage.setFileList(encryptedList);
					}
					// set first image:
					byte[] cipherBytes = ReadResources.readExternFile(encryptedList.get(0));
					if (cipherBytes == null) {
						new UnexpectedValueException("cipherBytes", "byte[]", "is null for: " + encryptedList.get(0)).printDescription();
					}
					// decrypt in RAM:
					byte[] plainBytes = CipherStuff.getInstance().decrypt(cipherBytes, CipherStuff.getSessionKeyCrypt().getKey(), true );
					if (plainBytes == null) {
						new UnexpectedValueException("plainBytes", "byte[]", "is null for: " + encryptedList.get(0)).printDescription();
					}
					TestLog.v(getClass(), "set Image: " + encryptedList.get(0));
					LockFrameImage.setImageBytes(plainBytes);
					if (lockFrame != null) {
						lockFrame.setMessage(PeaProperties.getVmBridge().translate("add_new_file")
								+ " - " + PeaProperties.getVmBridge().translate("done") 
								+ ": " + encryptedList.get(0));
					}
				} finally {
					// reset the cursor:
					win.setCursor(CursorManager.getDefaultCursor());
				}
			}  
		} else {
			return PeaProperties.getVmBridge().translate("no_valid_file_found");
		}
		if (errorBuilder.length() > 0) {
			return new String(errorBuilder);
		} else {
			return null;
		}
	}

	@Override
	public void openEncryptedFiles() {
		
		boolean fileAdded = AttachmentHandler.addEncryptedFiles(JREProperties.getMainWindow(), 
				PeaProperties.getVmBridge().translate("open_encrypted_file"), 
				true, false);// hidden files, do not check listed files
		
		if (fileAdded == false) { // no file was added
			System.err.println("no file was added");
			return;
		}

		
		boolean validFileFound = FilePanelHolder.getSelectedModel().checkForValidFile();
		if (validFileFound == false) {
			//PswDialogView.setMessage(PeaProperties.getOsBridge().translate("no_valid_file_found"), true);
			PswDialogView.setMessage(PeaProperties.getVmBridge().translate("open_or_initialize"), true, false);
		} else {
			// clear errorMessageLabel
			PswDialogView.setMessage("   ", true, false);
		}		
	}

	@Override
	public String getVersion() {
		return VERSION;
	}
	@Override
	public String getYearOfPublication() {
		return YEAR_OF_PUBLICATION;
	}
	@Override
	public String getWebsite() {
		return WEBSITE;
	}
	@Override
	public String getSourceLink() {
		return SOURCE_LINK;
	}

	@Override
	public boolean checkFormat(File file, boolean easyCheck, boolean middleCheck, boolean expensiveCheck) {
		
		return ImageHelper.checkImage(file, easyCheck, middleCheck, expensiveCheck);
	}

	@Override
	protected byte[] getPlainBytes() {
		TestLog.ve(ImageControl.class, "Call to not implemented method", 5);
		// not used 
		return null;
	}

	@Override
	public String setPeaPropValues(Properties prop, InputStream inputStream) {
		TestLog.ve(ImageControl.class, "Call to not implemented method", 5);
		return null;
	}

	@Override
	public String updatePeaPropValues(Properties prop) {//, InputStream inputStream) {
		TestLog.ve(ImageControl.class, "Call to not implemented method", 5);
		return null;
	}
	
	private static class  EventDispatchThread implements Runnable{

		@Override
		public void run() {
			PswDialogView.setUI(true);

			pswDialog = ImageControl.getInstance();
			ImageControl.dialogView.setVisible(true);
			setDialog(pswDialog);	
			if (FilePanelHolder.getFileNumber() == 0) {
				PswDialogView.setMessage(PeaProperties.getVmBridge().translate("open_or_initialize"), true, false);
			}
			if (new File("resources" + File.separator + "pea.properties").exists() == false) { // first time
				HtmlFileViewer hfv = new HtmlFileViewer("first_time_guide", dialogView);
				hfv.requestFocus();
			}			
		}		
	}

	@Override
	public String getPeaName() {
		return PEA_NAME;
	}

	@Override
	public String[] getSupportedExtensions() {
		return supportedExtensions;
	}
}