package cologne.eck.image_pea;

import java.awt.Color;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Main view of image pea. 
 */

/*
 * Frame to display images, decrypted in ImageControl
 * (this is not a daughter class of ContentUtilities!)
 * 
 */



import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
//import java.awt.image.ImageObserver;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.accessibility.AccessibleContext;
//import javax.activation.MimetypesFileTypeMap;
import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.StorageProvider;
import cologne.eck.all_peas.cloud.gui.CloudMenu;
import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.CloudFilePanel;
import cologne.eck.all_peas.files.FilePanelHolder;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.MultiFileSelectionDialog;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PeaLockFrame;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.gui.SingleFileSelectionDialog;
import cologne.eck.all_peas.gui.menu.HelpMenu;
import cologne.eck.all_peas.gui.menu.PeaMenuBar;
import cologne.eck.all_peas.gui.menu.SettingMenu;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.tools.PropertyHandler;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.WriteResources;
import cologne.eck.tools.Zeroizer;


@SuppressWarnings("serial")
final class LockFrameImage extends PeaLockFrame 
	implements ActionListener,// ImageObserver, 
	WindowListener {	
	//, AdjustmentListener {
	
	/**
	 * All file names of the image folder
	 */
	private static ArrayList <String> fileList;
	
	/**
	 * The index of the current image
	 */
	private int index = 0; 
	
	// the loaded image as an array of bytes
	private static byte[] imageBytes;
	
	// the image to show
	private static BufferedImage image = null;		
	// scroll pane containing the image
	private JScrollPane scrollPane = null;
	// vertical ScrollBar
	private JScrollBar verticalBar = null;//scrollPane.getVerticalScrollBar();
	// the label containing the scroll pane
	private JLabel picLabel = null;
	// label to show error and info messages 
	private JLabel nameLabel = null;
	
	/*
	 * The label for messages and errors
	 */
	private static JLabel messageLabel;


	// zoom value for width: additional width
	private static int zoomWidth = 0;
	// zoom value for height: additional height
	private static int zoomHeight = 0;
	
	// resized default width from ImageControl
	private int imgWidth = 0;
	// resized default height from ImageControl
	private int imgHeight = 0;
	
	private String[] arrayToChangePassword = null;

	
	private LockFrameImage(Point location) {
		
		PswDialogView.setUI(false);
		
	    this.setIconImage(PswDialogView.getImage());
	    
	    this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		
		this.addWindowListener(this);// for windowClosing
		
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		this.setContentPane(contentPane);		
		
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));
		menuPanel.setMaximumSize(new Dimension(600, 30));
		menuPanel.setMinimumSize(new Dimension(300, 30));
		
		PeaMenuBar menuBar = new PeaMenuBar();
		//menuBar.setBorder(new EtchedBorder() );
		
		JMenu menu = new JMenu();
		menu.setText(PeaProperties.getVmBridge().translate("file"));//"Menu");
		menu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("file_mnemonic")).getKeyCode());
		menuBar.add(menu);
		
		JMenuItem addNewItem = new JMenuItem();
		addNewItem.setText(PeaProperties.getVmBridge().translate("add_new_file"));
		addNewItem.setActionCommand("addNewFile");
		//encryptItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//addNewItem.setMnemonic(KeyEvent.VK_E);
		addNewItem.addActionListener(this);
		menu.add(addNewItem);
		
		JMenuItem encryptItem = new JMenuItem();
		encryptItem.setText(PeaProperties.getVmBridge().translate("close_encrypted"));
		encryptItem.setActionCommand("encrypted");
		//encryptItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//encryptItem.setMnemonic(KeyEvent.VK_E);
		encryptItem.addActionListener(this);
		menu.add(encryptItem);

		JMenuItem unencryptItem = new JMenuItem();
		unencryptItem.setText(PeaProperties.getVmBridge().translate("close_unencrypted"));
		unencryptItem.setActionCommand("unencrypted");
		//unencryptItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//unencryptItem.setMnemonic(KeyEvent.VK_U);
		unencryptItem.addActionListener(this);
		menu.add(unencryptItem);

		JMenuItem pswChangeItem = new JMenuItem();
		pswChangeItem.setText(PeaProperties.getVmBridge().translate("change_password"));//"change password");
		pswChangeItem.setActionCommand("changePassword");
		//pswChangeItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//pswChangeItem.setMnemonic(KeyEvent.VK_P);
		pswChangeItem.addActionListener(this);
		menu.add(pswChangeItem);
		
		JMenuItem convertItem = new JMenuItem();
		convertItem.setText(PeaProperties.getVmBridge().translate("convert")
				+ " (" + PeaProperties.getVmBridge().translate("encrypted") + ")");//"change password");
		convertItem.setActionCommand("convert");
		convertItem.addActionListener(this);
		menu.add(convertItem);

		JMenuItem gotoItem = new JMenuItem();
		gotoItem.setText(PeaProperties.getVmBridge().translate("go_to"));
		gotoItem.setActionCommand("goto");
		//gotoItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//gotoItem.setMnemonic(KeyEvent.VK_G);
		gotoItem.addActionListener(this);
		menu.add(gotoItem);
		

		JMenuItem quitItem = new JMenuItem();
		quitItem.setText(PeaProperties.getVmBridge().translate("quit"));
		quitItem.setActionCommand("quit");
		//quitItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//quitItem.setMnemonic(KeyEvent.VK_G);
		quitItem.addActionListener(this);
		menu.add(quitItem);
		
		JMenu settingMenu = new SettingMenu(this);
		menuBar.add(settingMenu);
		
		JMenu cloudMenu = new CloudMenu(this);
		menuBar.add(cloudMenu);

		JMenu helpMenu = new HelpMenu(this);
		menuBar.add(helpMenu);

		menuPanel.add(menuBar);
		menuPanel.add(Box.createHorizontalGlue());
		contentPane.add(menuPanel);
		
		JPanel buttonFileNamePanel = new JPanel();
		buttonFileNamePanel.setLayout(new BoxLayout(buttonFileNamePanel, BoxLayout.X_AXIS));
//		messagePanel.setPreferredSize(new Dimension (300, 30));
		buttonFileNamePanel.setMaximumSize(new Dimension(
				(int)PswDialogView.getScreenwidth()-100, 50));
		buttonFileNamePanel.setMinimumSize(new Dimension (300, 30));
		
		JButton previousButton = new JButton();//previous");
		Image previousImage = null;
		URL url = this.getClass().getResource("resources/zurueck.png");
		if (url == null) {
			try{
				previousImage = new ImageIcon(getClass().getClassLoader().getResource("resources/zurueck.png")).getImage();
			} catch (Exception e) {
				//TestLog.o(getClass(), "image not found " + e.getMessage() + ", " + e.toString());
			}
		} else {			
			previousImage = new ImageIcon(url).getImage();
		}
		if (previousImage != null) {
			previousButton.setIcon(new ImageIcon(previousImage));
		} else { // set characters
			previousButton.setText(" <-");
			previousButton.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 15));
		}
		previousButton.setMaximumSize(new Dimension(75,25));
		previousButton.setMargin(new Insets(1,1,1,1));
		previousButton.addActionListener(this);
		previousButton.setActionCommand( "previous");
		
		buttonFileNamePanel.add(previousButton);
		buttonFileNamePanel.add(Box.createHorizontalStrut(25));
		

		nameLabel = new JLabel(" Image PEA " + PeaControl.getDialog().getVersion());
		//messageLabel.setPreferredSize(new Dimension(300, 30));
		nameLabel.setMinimumSize(new Dimension(100, 30));
		nameLabel.setMaximumSize(new Dimension((int)PswDialogView.getScreenwidth() - 250, 30));
		buttonFileNamePanel.add(nameLabel);
		buttonFileNamePanel.add(Box.createHorizontalGlue());		
		
		JButton nextButton = new JButton();
		Image nextImage = null;
		url = this.getClass().getResource("resources/weiter.png");
		if (url == null) {
			try{
				nextImage = new ImageIcon(getClass().getClassLoader().getResource("resources/weiter.png")).getImage();
			} catch (Exception e) {
				//TestLog.o(getClass(), "image not found " + e.getMessage() + ", " + e.toString());
			}
		} else {			
			nextImage = new ImageIcon(url).getImage();
		}

		if (nextImage != null) {
			nextButton.setIcon(new ImageIcon(nextImage));
		} else { // set characters
			nextButton.setText("-> ");
			nextButton.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 15));
		}		
		//JButton nextButton = new JButton("next");
		nextButton.setMaximumSize(new Dimension(75,25));
		nextButton.setMinimumSize(new Dimension(75,25));
		nextButton.setMargin(new Insets(1,1,1,1));
		nextButton.addActionListener(this);
		nextButton.setActionCommand( "next");
		buttonFileNamePanel.add(nextButton);
		
		contentPane.add(buttonFileNamePanel);
		
		boolean success = setNewImage(imageBytes);
		Window win;
		if (PswDialogView.getView().isVisible()){
			win = PswDialogView.getView();
		} else {
			win = (JFrame) PeaControl.getDialog().getLockFrame();
		}
		if (success == false) {
			
			String decryptionMessage = "";
		
			if(CipherStuff.getErrorMessage() != null){
				decryptionMessage = PeaProperties.getVmBridge().translate(CipherStuff.getErrorMessage());
			}
			CipherStuff.setErrorMessage(null);

			if (fileList == null || fileList.size() == 0) {
				System.err.println("There a no files to load...");
				success = false;
			} else {
				String fileNameString = "";
				if (fileList.get(index) != null) {
					fileNameString = fileList.get(index);
					JOptionPane.showMessageDialog(win, 
							PeaProperties.getVmBridge().translate("cant_load_image")
							+ "\n   " + fileNameString
							+ "\n" + decryptionMessage, 
							PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
				}
				int len = fileList.size();
				if (len <= 1) {
					JOptionPane.showMessageDialog(win, 
							PeaProperties.getVmBridge().translate("cant_load_any_image"), 
							PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
					TestLog.ve(getClass(), "Close program...", 5);
					windowClosingCommands();
				} else {
					index++;
					while (success == false && index < len){
						success = loadImageByIndex(index);	
						index++;
					}
				}
			}
		}
		if (success == false) {
			JOptionPane.showMessageDialog(win, 
					PeaProperties.getVmBridge().translate("cant_load_any_image"),
					//+ "\n  " + PeaProperties.getVmBridge().translate("remove_and_try_other"), 
					PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
			TestLog.ve(getClass(), "Close program...", 5);
			windowClosingCommands();
		}
				
		// set maximum size, depending on screen size 
		scrollPane.setMaximumSize(new Dimension (
						(int)PswDialogView.getScreenwidth() - 100, 
						(int) PswDialogView.getScreenheight() - 300 ));// think of buttons
		// avoid page scroll for PDFs
		verticalBar = scrollPane.getVerticalScrollBar();
		verticalBar.setValue(5);

		// register scroll to begin and to end to render next pages
//		verticalBar.addAdjustmentListener(this);
		contentPane.add(scrollPane);
		
		JPanel messagePanel= new JPanel();
		messagePanel.setLayout(new BoxLayout(messagePanel, BoxLayout.LINE_AXIS));
		messagePanel.setMaximumSize(new Dimension((int) PswDialogView.getScreenwidth(), 30));
		messagePanel.setMinimumSize(new Dimension(200, 30));
		messageLabel = new JLabel();
		messagePanel.add(messageLabel);
		messagePanel.add(Box.createHorizontalGlue());
		contentPane.add(messagePanel);
		
		JPanel zoomPanel = new JPanel();
		zoomPanel.setMaximumSize(new Dimension(400, 30));
		zoomPanel.setMinimumSize(new Dimension(200, 30));
		
		JButton zoomInButton = new JButton ("+");
		zoomInButton.setActionCommand("zoomIn");
		zoomInButton.addActionListener(this);
		zoomPanel.add(zoomInButton);	
		
		JButton zoomOutButton = new JButton ("-");
		zoomOutButton.setActionCommand("zoomOut");
		zoomOutButton.addActionListener(this);
		zoomPanel.add(zoomOutButton);	
		
		zoomPanel.add(zoomInButton);
		zoomPanel.add(zoomOutButton);
		
		contentPane.add(zoomPanel);		

		
		this.setLocation(location);
		SwingUtilities.updateComponentTreeUI(this);
		
//		pack();
		int[] dim  = JREProperties.getFrameDim();
		if (dim[0] > PswDialogView.getScreenwidth()) {
			dim[0] = (int) PswDialogView.getScreenwidth();
		}
		if (dim[1] > PswDialogView.getScreenheight()) {
			dim[1] = (int) PswDialogView.getScreenheight();
		}

		this.setSize(new Dimension(dim[0], dim[1]));
		
		// pack or setMaximumSize/MinimumSize depending on screen size and image size
/*		if (this.getPreferredSize().getWidth() > PswDialogView.getScreenwidth() - 100
			|| this.getPreferredSize().getHeight() > PswDialogView.getScreenheight() - 200) {
			
			this.setMaximumSize(
					new Dimension (
							(int)PswDialogView.getScreenwidth() - 100,
							(int)PswDialogView.getScreenheight() - 200));
			int w = ImageControl.getImageWidth();
			if (w > PswDialogView.getScreenwidth() - 100){
				w = (int) (PswDialogView.getScreenwidth() - 100);
			}
			int h = ImageControl.getImageHeight();
			if (h > PswDialogView.getScreenheight() - 200){
				h = (int) (PswDialogView.getScreenheight() - 200);
			}
			// is is the shown size if the screen is large enough
			this.setMinimumSize(new Dimension (w, h));			
		} */
	}
	/**
	 * Returns an already instantiated instance 
	 * 
	 * @return 
	 */
	final static LockFrameImage getInstance(Point loc) {		
		
		
		if (PeaControl.getDialog().getLockFrame() == null) {
			return new LockFrameImage(loc);
		} else {
			return (LockFrameImage) PeaControl.getDialog().getLockFrame();
		}
	}
	 // just to test the view	 
/*	public static void main(String[] args) {

		PswDialogView.setUI(false);
		

//		File testFile = new File("resources/Phahlbaudorf.JPG");
		File testFile = new File("resources/images/drop.png");
		fileList = new ArrayList<String>();
		String imageFolder = "resources/images";
		File folder = new File(imageFolder);
		File[] files = folder.listFiles();
		int len = files.length;
		for(int i = 0; i < len; i++) {
			fileList.add(files[i].getAbsolutePath());
			TestLog.o(getClass(), files[i].getAbsolutePath());
		}
		LockFrameImage.setImageBytes(ReadResources.readExternFile(testFile.getAbsolutePath()));

		LockFrameImage frame = new LockFrameImage(new Point(100,100));
		frame.messageLabel.setText(testFile.getAbsolutePath());
		
		frame.setVisible(true);
	} */
	


	/**
	 * Zoom in or out: Resize the image, display the image and set the scroll bars 
	 * 
	 * @param in	if true: zoom in, otherwise zoom out
	 */
	private final void zoom(boolean in){
		
		  int previouseV = scrollPane.getVerticalScrollBar().getValue();
		  int previouseH = scrollPane.getHorizontalScrollBar().getValue();
		
		if (in == true) {			

			zoomWidth += imgWidth/2;
			zoomHeight += imgHeight/2;
			
		} else {
			
			zoomWidth -= imgWidth/2;
			zoomHeight -= imgHeight/2;	
			
			if (imgWidth + zoomWidth <= 0 || imgHeight + zoomHeight <= 0){
				//TestLog.o(getClass(), "Zoom out: invalid size values for image.");				
				zoomWidth += imgWidth/2;
				zoomHeight += imgHeight/2;
				// avoid IllegalArgumentException: do not paint the image
				return;
			}
		}
		try {
			// new compressed/resized image
			BufferedImage  compressedImage = new BufferedImage(
					imgWidth + zoomWidth, imgHeight + zoomHeight,
                  BufferedImage.TYPE_INT_ARGB);
            Graphics2D g = compressedImage.createGraphics();
            g.drawImage(image, 0, 0, imgWidth + zoomWidth, imgHeight + zoomHeight, this);              
            g.dispose();		
		  
             
            picLabel.setIcon(new ImageIcon(compressedImage));
               
      		if (in == true) {      	
                scrollPane.getVerticalScrollBar().setValue((int) (previouseV + imgHeight/4));
                scrollPane.getHorizontalScrollBar().setValue((int) (previouseH + imgWidth/4));                
      		} else {
                scrollPane.getVerticalScrollBar().setValue((int) (previouseV - imgHeight/4));
                scrollPane.getHorizontalScrollBar().setValue((int) (previouseH - imgWidth/4));
      		}
		} catch (IllegalArgumentException e){  // width and height are < 0

			if (PeaProperties.isTestMode()) { // test mode
				TestLog.o(getClass(), e.toString() + ", " + e.getMessage());
			}		 
			// reset the zoom
			if (in == true) {
				 zoomWidth -= ImageControl.getImageWidth() / 2;
				 zoomHeight -= ImageControl.getImageHeight() / 2;
			} else {
				 zoomWidth += ImageControl.getImageWidth() / 2;
				 zoomHeight += ImageControl.getImageHeight() / 2;		
			}
		} catch (OutOfMemoryError ex) { // ... if the image is too large
			if (PeaProperties.isTestMode()) { // test mode
				System.err.println(ex.toString() + " - " + ex.getMessage());
			  
				System.err.println("total memory: " + Runtime.getRuntime().totalMemory());
				System.err.println("free memory: " + Runtime.getRuntime().freeMemory());
				// used space: image width * image height * 8 (TYPE_INT_ARGB) 
				System.err.println("used space for image: " + ((imgWidth + zoomWidth) * (imgHeight + zoomHeight) * 8));
			}		  
			zoomWidth -= imgWidth/2;
			zoomHeight -= imgHeight/2;
		}
	}


	/**
	 * Set the image to show. This class holds a copy of the image as an array of bytes
	 * 
	 * @param input		the image as an array of bytes
	 */
	final static void setImageBytes( byte[] input){//, String fileName ) {
		if (input == null) {
			new UnexpectedValueException("input", "byte[]", "is  null").printDescription();
			return;
		}
		imageBytes = new byte[input.length];
		System.arraycopy(input,  0,  imageBytes,  0,  input.length);
	}
	
	/**
	 * Draws an image in the panel, shows an error
	 * message if fails and packs. 
	 * 
	 * @param imageBytes	the image to draw as byte array
	 * 
	 * @return	true, if the image was drawn successfully, 
	 * 			false otherwise (image should be removed then from list)
	 */
	private final boolean setNewImage(byte[] imageBytes) { //, String fileName) {
		
		if (imageBytes == null) { // was set to null before
			if (PeaProperties.isTestMode()) { // test mode	
				//new UnexpectedValueException("imageBytes", "byte[]", "is null").printDescription();
				TestLog.e(getClass(), "missing image");
				TestLog.e(getClass(), "imageBytes is null", 5);
			}
			return false;
		}
		// reset zoom 
		zoomWidth = 0;
		zoomHeight = 0;

		// get image from file name:		
		try {					
			image = ImageIO.read(new ByteArrayInputStream(imageBytes));

		} catch (IOException e) {			
			System.err.println("LockFrameImage: " + e.toString() + " - " + e.getMessage());
			if (PeaProperties.isTestMode()) { // test mode				
				e.printStackTrace();
			} 
		}
		Window win;
		if (PswDialogView.getView().isVisible()){
			win = PswDialogView.getView();
		} else {
			win = (JFrame) PeaControl.getDialog().getLockFrame();
		}
		if (image == null) { // invalid file
			JOptionPane.showMessageDialog(win, 
					PeaProperties.getVmBridge().translate("invalid_image_file") + "\n" //"Invalid or unsuitable image file: \n"                      
							+ fileList.get(index),
//							+ ImageControl.getEncryptedFileName(), 
					      PeaProperties.getVmBridge().translate("error"),                                
					   JOptionPane.ERROR_MESSAGE);   
			return false;
		}
		// resize the image
		int[] newSize = ImageHelper.resizeImage(image, ImageControl.getImageWidth(), ImageControl.getImageHeight());
		imgWidth = newSize[0];
		imgHeight = newSize[1];	
		
		// check if image is larger than default values for width and height: 
		if (image.getWidth() > imgWidth || image.getHeight() > imgHeight){

			// compress the image:
			BufferedImage  resizedImage = new BufferedImage(imgWidth, imgHeight,
	        BufferedImage.TYPE_INT_ARGB);
	        Graphics2D g = resizedImage.createGraphics();
	        g.drawImage(image, 0, 0, imgWidth, imgHeight, this);              
	        g.dispose();	

	        if (picLabel== null) {
	        	picLabel= new JLabel();
	        }
			picLabel.removeAll();
			picLabel.setIcon(new ImageIcon(resizedImage));
//			scrollPane = new JScrollPane(picLabel);
			if (scrollPane == null) {
				scrollPane = new JScrollPane(picLabel);
			}
			scrollPane.setPreferredSize(new Dimension(imgWidth, imgHeight));
		} else {
			// do not compress
	        if (picLabel== null) {
	        	picLabel= new JLabel();
	        }
			picLabel.removeAll();
			picLabel.setIcon(new ImageIcon(image));

			if (scrollPane == null) {
				scrollPane = new JScrollPane(picLabel);
			}
			scrollPane.setPreferredSize(new Dimension(imgWidth, imgHeight));
			imgWidth = image.getWidth();
			imgHeight = image.getHeight();
		}
		//currentFileName = fileName;
		
//		this.pack();
		return true;
	}
	
	public void setMessage(String _message) {

		nameLabel.setText(_message);
	}

	@Override
	public void actionPerformed(ActionEvent ape) {

		String command = ape.getActionCommand();

		if (command.equals("unencrypted")) {
			
			MultiFileSelectionDialog dialog = new MultiFileSelectionDialog(
					this, this.getLocationOnScreen(), 
					PeaProperties.getVmBridge().translate("select_files_remain_unencrypted"),
					fileList);
			if (dialog.getSelectedFiles() == null) {				
				return;
			}
			ArrayList<String> unencryptedList = dialog.getSelectedFiles();
			dialog.dispose();
			
			if (unencryptedList == null || unencryptedList.size() == 0) {
				return;
			}
			// extra confirmation:
			String selectedList = "";
			for (String s : unencryptedList) {
				if (s != null) {
					selectedList += s + "\n";
				}
			}
			String cloudHint = "";
			if (selectedList.contains(CloudControl.getTmpDir()) == true) {
				cloudHint = "\n\n" + PeaProperties.getVmBridge().translate("cloud_files_in_folder") + "\n" 
						+ CloudControl.getTmpDir();
			}
			int leaveResult = PeaDialog.showQuestion(this, 
					PeaProperties.getVmBridge().translate("leave_files_unencrypted")
					+ "\n\n" + selectedList
					+ cloudHint,
					PeaProperties.getVmBridge().translate("warning"), 0);
			if (leaveResult != 0) {
				TestLog.v(getClass(), "Do not leave files unencrypted...");
				return;
			}
			try {
				// set wait cursor
				this.setCursor(CursorManager.getWaitCursor());
				nameLabel.setText(PeaProperties.getVmBridge().translate("decryption") + " " + PeaProperties.getVmBridge().translate("in_progress"));
				// the current displayed file:
				// the index will change because elements will be removed
				String currentFileName = fileList.get(index);
				
				int len = unencryptedList.size();
				int fileListLen = fileList.size();
				boolean closeCurrentImage = false;
				ArrayList<String> removedList = new ArrayList<String>();
				for (int i = 0; i < len; i++) {
					if ( ! new File(unencryptedList.get(i)).isFile()) {
						// directory
						// set null in fileList to mark as invalid
						for (int j = 0; j < fileListLen; j++) {
							if (fileList.get(j).equals(unencryptedList.get(i))) {
								TestLog.ve(getClass(), "Set file name to null: " + fileList.get(j));
								fileList.set(j,  null);
							}
						}
						continue;
					}
						
					if (unencryptedList.get(i) == null) {
						System.err.println("File name is null at index " + i);
						continue;
					}
						
					if (unencryptedList.get(i).equals(currentFileName)) { // current shown image:
						closeCurrentImage = true;
						// close at least
						continue;
					}
					//read cipherBytes, decrypt and store
					byte[] cipherBytes = ReadResources.readExternFile(unencryptedList.get(i));
					byte[] plainBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );
					if (plainBytes == null) {
						System.err.println("Decryption failed at index " + i
								+ " for file name " + unencryptedList.get(i));
						JOptionPane.showMessageDialog((Window)PeaControl.getDialog().getLockFrame(), 
								PeaProperties.getVmBridge().translate("decryption_failed")
								+ "\n" + unencryptedList.get(i)
								+ PeaProperties.getVmBridge().translate(CipherStuff.getErrorMessage()), 
								PeaProperties.getVmBridge().translate("error"), 
								JOptionPane.ERROR_MESSAGE);
						CipherStuff.setErrorMessage(null);
						// do not store in path file
						// set null in fileList to mark as invalid
						for (int j = 0; j < fileListLen; j++) {
							if (fileList.get(j).equals(unencryptedList.get(i))) {
								TestLog.ve(getClass(), "Set file name to null: " + fileList.get(j));
								fileList.set(j,  null);
							}
						}
						
						PeaControl.getAvailableFileNames().remove(unencryptedList.get(i));
						continue;
					} else {
						WriteResources.write(plainBytes, unencryptedList.get(i), null );
						removedList.add( unencryptedList.get(i));
					}
				}
				// remove from available files and remembered files
				ArrayList<String> availabelFileNames = PeaControl.getAvailableFileNames();
				if ( removedList != null && removedList.isEmpty() == false) {
					availabelFileNames.removeAll(removedList);		
					fileList.removeAll(removedList);
					for (String fileNameToRemove : removedList) {
						PathFileManager.removeFileName(fileNameToRemove);
					}
				}
				PeaControl.setAvailableFileNames(availabelFileNames);
				
				if (closeCurrentImage == true ) {
					//read cipherBytes, decrypt and store
					byte[] cipherBytes = ReadResources.readExternFile(currentFileName);
					byte[] plainBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );
					if (plainBytes == null) {
						System.err.println("Decryption failed for " + currentFileName);
						JOptionPane.showMessageDialog((Window) PeaControl.getDialog().getLockFrame(), 
								PeaProperties.getVmBridge().translate("decryption_failed")
								+ "\n" + currentFileName
								+ PeaProperties.getVmBridge().translate(CipherStuff.getErrorMessage()),
								PeaProperties.getVmBridge().translate("error"), 
								JOptionPane.ERROR_MESSAGE);
						CipherStuff.setErrorMessage(null);
						// do not store in path file
						ArrayList<String> availableFileNames = PeaControl.getAvailableFileNames();
						availableFileNames.remove(currentFileName);
						PeaControl.setAvailableFileNames(availableFileNames);
						for (int j = 0; j < fileListLen; j++) {
							if (fileList.get(j).equals(currentFileName)) {
								TestLog.ve(getClass(), "Set file name to null: " + fileList.get(j));
								fileList.set(j,  null);
							}
						}
						TestLog.ve(getClass(), "Close program...", 5);
						windowClosingCommands();
					} else {
						PathFileManager.removeFileName(currentFileName);
						fileList.remove(currentFileName);
					}
						
					WriteResources.write(plainBytes, currentFileName, null );						
					// give message:
					JOptionPane.showMessageDialog(this,
							PeaProperties.getVmBridge().translate("current_file_closed"));
					// close program: 
					Zeroizer.zero(imageBytes);
					//PeaControl.setAvailableFileNames(null);
					//fileList.clear();
					TestLog.ve(getClass(), "Close program...", 5);
					windowClosingCommands();
					System.exit(0);
				}

			} finally {
				this.setCursor(CursorManager.getDefaultCursor());
				nameLabel.setText("");
			}
			// only close program if current file was selected
			//System.exit(0);
			
		} else if (command.equals("encrypted")){
			// file was not modified, just loaded unencrypted in RAM
			PathFileManager.addFileNamesToPathFile(fileList);
			Zeroizer.zero(imageBytes);
			TestLog.ve(getClass(), "Close program...", 5);
			windowClosingCommands();
			
		} else if (command.equals("addNewFile")){
			
			PeaControl.getDialog().initializeNewFiles();
			
		} else if (command.equals("convert")){
			TestLog.o(getClass(), "Convert current image: " + fileList.get(index));
			
			// get the output format:
			String[] formats = PeaControl.getDialog().getSupportedExtensions();		
			int resultIndex = PeaDialog.showOptions(this, PeaProperties.getVmBridge().translate("format") + ": \n", 
					PeaProperties.getVmBridge().translate("convert"), 
					2, -1, 
					formats, 0);		
			if (resultIndex == -1) { // cancelled
				return;
			}
			byte[] result = ImageHelper.convertImage(image, formats[resultIndex]);
			if (result == null ) {
				PeaDialog.showMessage(this, 
						PeaProperties.getVmBridge().translate("convert") + "  ->  " + formats[resultIndex] 
								+ "\n" + PeaProperties.getVmBridge().translate("failed"), 
						PeaProperties.getVmBridge().translate("error"), 0);
				return;
			}
			byte[] newImageBytes = result;
			String format = formats[resultIndex];

			if (newImageBytes != null) {
				// choose location to store, encrypt and save in file
				FileFilter imageFilter = new FileNameExtensionFilter(
						"Image files", format);
				String fn = fileList.get(index);
				String suggestedFileName = fn.substring(0, fn.lastIndexOf(".")) + "." + format;
				PeaFileChooser chooser = new PeaFileChooser(this);
				chooser.setFileFilter(imageFilter);
				chooser.setMultiSelectionEnabled(false);
				chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				chooser.setSelectedFile(new File(suggestedFileName));
				chooser.customizeFileChooser(
						null,//PeaProperties.getVmBridge().translate("select_images_to_encrypt"),
						PeaProperties.getVmBridge().translate("select_single_image_or_folder"),
						false);// no hidden files option
				
				int returnVal = chooser.showOpenDialog(this);
				
				File file = chooser.getSelectedFile();
				if (file.exists() == true) {
					int n = PeaDialog.showQuestion(this, 
							PeaProperties.getVmBridge().translate("overwrite_existing_file"), 
							PeaProperties.getVmBridge().translate("warning"), 2);
					if (n != 0) { // cancel, close
						TestLog.v(getClass(), "Overwriting cancelled");
						return;
						// System.exit(0);
					}
				}

				if(returnVal == JFileChooser.APPROVE_OPTION) {	// ok
					byte[] cipherBytes = new CipherStuff().encrypt(newImageBytes, null, true, null);
					String fileName = file.getAbsolutePath();
					boolean writeSuccess = WriteResources.write(cipherBytes, fileName, null);
					if (writeSuccess == false) {
						PeaDialog.showMessage(this, PeaProperties.getVmBridge().translate("failed") + ":\n"
								+ fileName + "\n(Could not write file)", 
								PeaProperties.getVmBridge().translate("error"), 0);
					} else {
						PeaDialog.showMessage(this, PeaProperties.getVmBridge().translate("success") + ":\n"
								+ fileName );

						if (file.getAbsolutePath().contains(CloudControl.getTmpDir())) {
							try {
								ContentUtilities.saveTmpFileInCloud(fileName);
								String providerName = CloudControl.getProviderNameFromTmpFile(fileName);
								String fileNameWithoutPath = CloudControl.getFileNameFromTmpFile(fileName);
								if (providerName != null && fileNameWithoutPath != null && 
										FilePanelHolder.getSelectedPanel() != null && 
										FilePanelHolder.getSelectedPanel().getCloudFilePanel() != null) {
									CloudControl cc = FilePanelHolder.getSelectedPanel().getCloudFilePanel().getCloudControl();
									cc.addUsedFileNameToProvider(providerName, fileNameWithoutPath);
									ContentUtilities.saveTmpFileInCloud(fileName);
								}
							} catch (Exception e) {
								PeaDialog.showMessage(this, 
										"In temporary directory: \n " + fileName, PeaProperties.getVmBridge().translate("warning"), 2);
							}
						}
					}
				} else {
					Zeroizer.zero(newImageBytes);
					return;
				}				
				// Add to list
				addFileToList(chooser.getSelectedFile().getAbsolutePath());
		
				
			} /*else {
				PeaDialog.showMessage(this, 
						PeaProperties.getVmBridge().translate("failed") + ": \n" 
						+ PeaProperties.getVmBridge().translate("convert")
						+ " " + fileList.get(index), 
						PeaProperties.getVmBridge().translate("error"), 0);
			}*/
			
		} else if (command.equals("changePassword")){
		
			arrayToChangePassword = chooseFileNamesToApply();
			if (arrayToChangePassword == null) {
				return;
			}
			String changeError = this.changePasswordOrApplyChanges(this, arrayToChangePassword, false);
			if (changeError != null) {
				PeaDialog.showMessage(this, changeError, 
						PeaProperties.getVmBridge().translate("error"), 0);
			} else {
				TestLog.o(getClass(), "Successfully changed password/algorithms ");
//TestLog.v(getClass(), "Used scheme: " + AlgoParamHandler.writeString(false).getBytes(AttachmentHandler.getASCIICharset()));
				this.setChangesApplied(true);
			}

		} else if (command.equals("zoomIn")){
			zoom(true);
		} else if (command.equals("zoomOut")){
			zoom(false);
		} else if (command.equals("about")) {
			PswDialogView.showAboutDialog("Image Lock PEA", PeaProperties.getVmBridge().translate("supported_formats") + ": png, jpg, gif, wbmp", this);
			
		} else if (command.equals("infos")){			
			
			PswDialogView.showInfoDialog(this);
		} else if (command.equals("previous")){				
			
			int newIndex = 0;
			if (index == 0) { // begin...
				// show last image:
				newIndex = fileList.size() - 1;
			} else {
				// show previous image:
				newIndex = index - 1;
			}
			boolean success = false;
			String rememberedFileName = fileList.get(newIndex);// is set to null if loadOtherImage fails 
			
			try {
				// set wait cursor
				this.setCursor(CursorManager.getWaitCursor());			
			
				while (newIndex != index ) { // index is only updated for success
					// end of loop if no valid image is found
					if (newIndex < 0) {
						newIndex = fileList.size() - 1;						
					}
	
					if (fileList.get(newIndex) == null) {
						// was marked as invalid
						newIndex--;
						continue;
					} else {
						// try to load previous image:
						rememberedFileName = fileList.get(newIndex);// is set to null if loadOtherImage fails 
						success = loadImageByIndex(newIndex);
					}					
					if (success == true) {
						this.setCursor(CursorManager.getDefaultCursor());
						return;
					} else {
						String errorString = CipherStuff.getErrorMessage();
						try {
							errorString = PeaProperties.getVmBridge().translate(errorString);
						} catch (Exception e) {
							//new UnexpectedValueException("errorString", "String", "is not key in language bundle: " + errorString).printDescription();
						}
						JOptionPane.showMessageDialog(this, 
								PeaProperties.getVmBridge().translate("cant_load_image")
								+ ": \n" + rememberedFileName
								+ "\n" + errorString,
								PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
						// reset wait cursor because of modal dialog
						this.setCursor(CursorManager.getWaitCursor());
						CipherStuff.setErrorMessage(null);

						// try image before previous image
						newIndex--;
						if (newIndex < 0) { // begin...
							// show last image:
							newIndex = fileList.size() - 1;
						}
					}
				}
				if (success == false) {
					
					// try to load current image again:
					rememberedFileName = fileList.get(newIndex);// is set to null if loadOtherImage fails 
					success = loadImageByIndex(index);
					if (success == false) {
						System.err.println("Can't load " + rememberedFileName);
						String errorString = CipherStuff.getErrorMessage();
						try {
							errorString = PeaProperties.getVmBridge().translate(errorString);
						} catch (Exception e) {
							new UnexpectedValueException("errorString", "String", "is not key in language bundle: " + errorString).printDescription();
						}
						JOptionPane.showMessageDialog(this, 
								PeaProperties.getVmBridge().translate("cant_load_any_image")
								+ "\n" + errorString,
								PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
						TestLog.ve(getClass(), "Close program...", 5);
						windowClosingCommands();
					}
				}
			} finally {
				this.setCursor(CursorManager.getDefaultCursor());
			}
			
		} else if (command.equals("next")){		
			
			// register loading and decryption sucess of new image
			boolean success = false;
			// index of the image to show next
			int newIndex = 0;
			if (index + 1 >= fileList.size()) { // end...
		    	// set first image:
		    	newIndex = 0;
			} else {
				// set next image:
				newIndex = index + 1;
			}
			String rememberedFileName = fileList.get(newIndex);// is set to null if loadOtherImage fails 
			try {
				// set wait cursor
				this.setCursor(CursorManager.getWaitCursor());
						
				while (newIndex != index) { // index is only updated for success
					// end of loop if no valid image is found
					if (newIndex >= fileList.size()) { // end...
				    	// set first image:
				    	newIndex = 0;
				    	if (newIndex == index) {
				    		// no other valid image found
							JOptionPane.showMessageDialog(this, 
									PeaProperties.getVmBridge().translate("cant_load_other_image"), 
									PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
							// reset wait cursor because of modal dialog
							this.setCursor(CursorManager.getWaitCursor());
							return;
				    	}
					}
			
					if (fileList.get(newIndex) == null) {
						// image is marked as invalid...
						newIndex++;
						continue;
					} else {					
						if (new File(fileList.get(newIndex)).isDirectory()) {
							//fileList.remove(newIndex);
							newIndex++;
							continue;
						}
						// try to load next image:
						rememberedFileName = fileList.get(newIndex);// is set to null if loadOtherImage fails 
						success = loadImageByIndex(newIndex);
					}	
					
					if (success == true) {
						this.setCursor(CursorManager.getDefaultCursor());
						return;
					} else {
						String errorString = CipherStuff.getErrorMessage();
						try {
							errorString = PeaProperties.getVmBridge().translate(CipherStuff.getErrorMessage());
						} catch (Exception e) {
							//new UnexpectedValueException("errorString", "String", "is not key in language bundle: " + errorString).printDescription();
						}
						JOptionPane.showMessageDialog(this, 
								PeaProperties.getVmBridge().translate("cant_load_image") + ": \n" + rememberedFileName
								+ "\n" + errorString,
								PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
						CipherStuff.setErrorMessage(null);
					}
					// reset wait cursor because of modal dialog
					this.setCursor(CursorManager.getWaitCursor());
					
					// try image after next image
					newIndex++;
			
				} // end of loop
				if (success == false) {
					// try to load current image again:
					success = loadImageByIndex(index);
					if (success == false) {
						JOptionPane.showMessageDialog(this, 
								PeaProperties.getVmBridge().translate("cant_load_any_image")
								+ "\n" + PeaProperties.getVmBridge().translate(CipherStuff.getErrorMessage()),
								PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
						TestLog.ve(getClass(), "Close program...", 5);
						windowClosingCommands();
					}
				}
			} finally {
				this.setCursor(CursorManager.getDefaultCursor());
			}
			
		}  else if (command.equals("goto")) {
			SingleFileSelectionDialog dialog = new SingleFileSelectionDialog(
					((Window)PeaControl.getDialog().getLockFrame()), 
					((Window) PeaControl.getDialog().getLockFrame()).getLocationOnScreen(), 
					PeaProperties.getVmBridge().translate("select_image_file"), 
					fileList);
			String fileName = SingleFileSelectionDialog.getSelectedFileName();
			dialog.dispose();
			String rememberedFileName = fileName;// is set to null if loadOtherImage fails 
			boolean success = loadImageByIndex(fileList.indexOf(fileName));
			if (success == false) {
				String error = CipherStuff.getErrorMessage();
				try { 
					error = PeaProperties.getVmBridge().translate(error);
				} catch (Exception e) {
					System.err.println("Can't get language String for " + error);
				}
				JOptionPane.showMessageDialog(this, 
						PeaProperties.getVmBridge().translate("cant_load_image") + ": \n" + rememberedFileName
						+ "\n"+ error, 
						PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
				CipherStuff.setErrorMessage(null);
				// load last image:
				success = loadImageByIndex(index);
				if (success == false) {
					JOptionPane.showMessageDialog(this, 
							PeaProperties.getVmBridge().translate("cant_load_any_image"), 
							PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
					TestLog.ve(getClass(), "Close program...", 5);
					windowClosingCommands();
				}
			} else {
				if (fileName.startsWith(CloudControl.getTmpDir())) {
					String name = CloudControl.getFileNameFromTmpFile(fileName);
					String providerName = CloudControl.getProviderNameFromTmpFile(fileName);
					nameLabel.setText(PeaProperties.getVmBridge().translate("cloud")
							+ " " + providerName + ": " + name);
				} else {
					nameLabel.setText(fileName);	
				}
			}
		}  else if (command.equals("quit")) {
			// Can always close the program even if exceptions occur
			try {
				this.windowClosingCommands();
			} catch (Exception e) {
				int result = PeaDialog.showQuestion(this, e.getLocalizedMessage() 
						+ "\n   " + PeaProperties.getVmBridge().translate("quit") + "?", 
						PeaProperties.getVmBridge().translate("error"), 0);
				if (result == 0) {
					System.exit(0);
				}
			}
		} 
	}

	/**
	 * Loads an image from the list, sets the image 
	 * to null if it fails
	 * 
	 * @param newIndex	the index of the image to load
	 * 
	 * @return	true: success, 
	 * 			false otherwise (image is set to null in this case)
	 */
	private boolean loadImageByIndex(int newIndex) {
		
		if (newIndex < 0 || newIndex >= fileList.size()) {
			new UnexpectedValueException("newIndex", "int", 
					"is not in range 0 - " + fileList.size() + ": " + newIndex).printDescription();
			return false;
		}
		
		if (fileList.get(newIndex) == null) {
			return false;
		}
		
		boolean success = true;
		byte[] newImageBytes = null;

		if ( new File(fileList.get(newIndex)).isFile()) {
			try {
				byte[] cipherBytes = null;
				if (fileList.get(newIndex).equals(PeaProperties.getVmBridge().translate("image_inside_jar_archive"))){
					cipherBytes = ReadResources.getResourceFromJAR("resources/text.lock");
				} else {
					cipherBytes = ReadResources.readExternFile(fileList.get(newIndex));
				}
				//currentFileName = fileList.get(fileList.size() - 1);
				newImageBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );

			} catch (Exception e) {
				// show the previous image:
				System.err.println(e.toString() + ", " + e.getMessage());
				success = false;
				// set element null to indicate error
				TestLog.ve(getClass(), "Set file name to null: " + fileList.get(newIndex));
				fileList.set(newIndex, null);	
				//fileList.remove(newIndex);
			}
		} else {
			// mark as invalid
			TestLog.ve(getClass(), "Set file name to null: " + fileList.get(newIndex));
			fileList.set(newIndex, null);
			//fileList.remove(newIndex);
		}
		if (newImageBytes != null) {
			// set newImageByte as current image
			imageBytes = newImageBytes;
			// load new image:
			boolean loadSuccess = setNewImage(imageBytes);
			if (loadSuccess == false){
				fileList.set(newIndex, null);	
				//fileList.remove(newIndex);
				return false;
			}
			String fileName = fileList.get(newIndex);
			if (fileName.startsWith(CloudControl.getTmpDir())) {
				String name = CloudControl.getFileNameFromTmpFile(fileName);
				String providerName = CloudControl.getProviderNameFromTmpFile(fileName);
				nameLabel.setText(PeaProperties.getVmBridge().translate("cloud")
						+ " " + providerName + ": " + name);
			} else {
				nameLabel.setText(fileName);	
			}
			// set new index as current index
			index = newIndex;
		} else {
			TestLog.ve(getClass(), "Image bytes is null", 5);
			TestLog.ve(getClass(), "Set file name to null: " + fileList.get(newIndex));
			fileList.set(newIndex, null);	
			success = false;
		}		
		return success;
	}
	
	@Override
	public void windowClosing(WindowEvent arg0) { // by X-Button
		windowClosingCommands();
	}
	@Override
	public void windowActivated(WindowEvent arg0) {}
	@Override
	public void windowClosed(WindowEvent arg0) {}
	@Override
	public void windowDeactivated(WindowEvent arg0) {}
	@Override
	public void windowDeiconified(WindowEvent arg0) {}
	@Override
	public void windowIconified(WindowEvent arg0) {}
	@Override
	public void windowOpened(WindowEvent arg0) {}
	
	private void windowClosingCommands() {

		// Check if algorithms or parameters were changed
		// but not yet applied: 
		// This is always immediately applied in Image Lock PEA for the chosen files
		/*		if (isChangesApplied() == false) {// 
			changePasswordOrApplyChanges(this, arrayToChangePassword, true);
			setChangesApplied(true);
		}*/



		// ********************* later version... **************	
		//PdfToPngConverter.closeDocument();



		// check if cloud processes are still running and ask to exit
		if (CloudControl.getNumberOfRunningCloudProcesses() > 0) {
			String[] options = {PeaProperties.getVmBridge().translate("cancel"),
					PeaProperties.getVmBridge().translate("ok")};


			int result = PeaDialog.showOptionYAxis(this, 
					PeaProperties.getVmBridge().translate("cloud_process_warning")
					+ "\n" + PeaProperties.getVmBridge().translate("terminating_damage"), 
					PeaProperties.getVmBridge().translate("warning"), 
					2,  options, 0);

			if (result == 1) {
				TestLog.v(this.getClass(), "Closing although cloud process is running...");
			} else {
				TestLog.v(this.getClass(), "Upload process is running.... Break exit...");
				return;
			}
		}
		// set location and dimension
		new PropertyHandler().updateAllPropValues();
		
		// ask to remember files:
		if (fileList != null && fileList.size() > 0) {
			// first remove null elements to avoid NullPointerExc.
			// use iterator to avoid ConcurrentModificationException
			Iterator<String> iti = fileList.iterator();
			while (iti.hasNext()) {
				String s = iti.next();
				if (s == null) {
					iti.remove();
				}
			}
			PathFileManager.addFileNamesToPathFile(fileList);
		}

		// zeroize current image
		if (imageBytes != null) {
			Zeroizer.zero(imageBytes);
		}
		// clear the sessionKeys:
		CipherStuff.getSessionKeyCrypt().clearKeys();	

		// Close all providers
		Collection<StorageProvider> openProviders = CloudControl.getOpenProviders();
		if (openProviders != null &&  ! openProviders.isEmpty()) {
			for (StorageProvider sp : openProviders) {
				String error = sp.closeSession();
				if (error != null) {
					TestLog.e(getClass(), error);
				}
			}
		}

		// delete all temporary backup files of cloud files:
		if (PeaControl.getDialog().getEncryptedFileTypePanel() != null) { // after initializing maybe null
			CloudFilePanel cfp = PeaControl.getDialog().getEncryptedFileTypePanel().getCloudFilePanel();
			if (cfp != null) {
				String[] providers = cfp.getCloudControl().getUsedProviders();
				if (providers != null) {
					for (String provName : providers) {	
						if (provName != null) {
							String[] filesOfProv = cfp.getCloudControl().getUsedFileNamesOfProvider(provName);
							if (filesOfProv != null) {
								for (String fileName : filesOfProv) {
									if (fileName != null && fileName.length() >0) {
										File fileToDelete = new File(CloudControl.getTmpFileNameForCloudFile(fileName, provName, null));
										if (fileToDelete.exists()) {
											boolean deleteResult = fileToDelete.delete();
											if (deleteResult == false) {
												TestLog.e(LockFrameImage.class, "Could not delete " + fileToDelete);
											} else {
											}
										} else {
											TestLog.e(LockFrameImage.class, "File form cloud to delete does not exist " + fileToDelete);
										}
									} else {
										TestLog.e(LockFrameImage.class, "Could find file to delete from cloud " + provName);
									}
								}
							} else {
								TestLog.e(LockFrameImage.class, "Could find file to delete from cloud... ");
							}
						} else {
							TestLog.e(LockFrameImage.class, "Cloud provider is null... ");
						}
					}
				}
			}
		}
		TestLog.o(this.getClass(), "Exit");
		System.exit(0);
	}


	/**
	 * Set an new list of image file names. Already existing list will be
	 * overwritten
	 * 
	 * @param newList	the new list
	 */
	static void setFileList(ArrayList <String> newList) {
		
		fileList = newList;
	}
	/**
	 * Get the list of image file names
	 * 
	 * @return	the list of image file names
	 */
/*	public static ArrayList<String> getFileList() {
		return fileList;
	}*/
	/**
	 * Add a file name to the list of image file names
	 * 
	 * @param newFileName	the file name of the image file
	 * 
	 * @retun null for success, an error message otherwise
	 */
	static String addFileToList(String newFileName) {
		if (newFileName == null) {
			TestLog.e(LockFrameImage.class, "No file to add ");
			return PeaProperties.getVmBridge().translate("unexpected_error") + " (no file)";
		}
		if ( new File(newFileName).isFile()) {
			if (fileList == null) {
				fileList = new ArrayList<String>();
			}
			if ( ! fileList.contains(newFileName)) {
				fileList.add(newFileName);
				return null;
			} else {
				TestLog.e(LockFrameImage.class, "list already contains file " + newFileName);
				return PeaProperties.getVmBridge().translate("file_already_open");
			}
		} else {
			TestLog.e(LockFrameImage.class, "file to add is not a file: " + newFileName);
			return PeaProperties.getVmBridge().translate("unexpected_error") + " (not a file)";
		}
	}

    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleLockFrameImage();
        }
        return accessibleContext;
    }

    protected class AccessibleLockFrameImage extends AccessibleJFrame {
        //Inherit everything, override nothing.
    }

/*	@Override
	public boolean imageUpdate(Image arg0, int arg1, int arg2, int arg3, int arg4, int arg5) {
		return false;
	}*/
	@Override
	public String[] getCurrentFileNames() {		
		String[] cfn = {fileList.get(index)};
		return cfn;
	}
	@Override
	public void setMessage(String message, boolean isError) {
		if(isError == true) {
			messageLabel.setForeground(Color.red);
			messageLabel.setText(message);			
		} else {
			messageLabel.setForeground(Color.black);
			messageLabel.setText(message);
		}
	}

	@Override
	public boolean checkUnsavedContentChanges() {
		TestLog.ve(getClass(), "Method not implemented...", 5);
		return false;
	}
	@Override
	public void markUnsavedContentChanges() {
		TestLog.ve(getClass(), "Method not implemented...", 5);
	}
	@Override
	public void markContentIsSaved() {
		//TestLog.ve(getClass(), "Method not implemented...", 5);
	}
	@Override
	public String saveCurrentContentInCloudTempFile(String providerName, String fileNameToStore) {
		TestLog.ve(getClass(), "Method not implemented...", 5);
		return null;
	}
	@Override
	public void setCurrentFileNames(String[] fileNameToSet) {
		new UnexpectedValueException("function","setCurrentFileNames","is not implemented").printDescription();
	}
	@Override
	public void updateModelWithCloudFiles(File[] files, String providerName) {
		//new UnexpectedValueException("function","updateModelWithCloudFiles","is not implemented").printDescription();
	}
	@Override
	public String decryptAndLoadFileToView(String fileName) {
		String error = addFileToList(fileName);
		return error;
	}
	@Override
	public String saveCurrentContent() {
		//TestLog.ve(getClass(), "Method not implemented...", 5);
		return null;
	}
	@Override
	public String[] chooseFileNamesToApply() {
		
		//TODO synchronized files are now listed separated
		// show a list of files
		MultiFileSelectionDialog dialog = new MultiFileSelectionDialog(
				this, this.getLocationOnScreen(), 
				PeaProperties.getVmBridge().translate("Select_file_for_password_change"),
				fileList);
		
		// the list of the selected files
		ArrayList<String> unencryptedList = dialog.getSelectedFiles();
		// check if current file is included:
		if (unencryptedList == null ||  ! (unencryptedList.contains(fileList.get(index))) ){
			JOptionPane.showMessageDialog(this, 
					PeaProperties.getVmBridge().translate("must_contain_current_file") +":\n" + fileList.get(index), 
					PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
			return null;
		} 
		dialog.dispose();			

		// Note: Wait cursor is set in ContentUtilities.changePassword
		//String[] arrayToChangePassword = null;
		if (unencryptedList != null && unencryptedList.size() > 0) {
			// create array from the list:
			arrayToChangePassword = new String[unencryptedList.size()];
			arrayToChangePassword = unencryptedList.toArray(arrayToChangePassword);
		}
		return arrayToChangePassword;
	}
}