package cologne.eck.peafactory.crypto.kdf;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import org.bouncycastle.crypto.generators.SCrypt;

import cologne.eck.tools.Comparator;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;

public class ScryptKDF extends KeyDerivation {
	
	private static int memoryFactor = 32; // r Memory cost parameter
	private static int cPUFactor = 16384;//16384; // N CPU cost parameter
	private static int parallelFactor = 1;// 1; // p Parallelization parameter

	public ScryptKDF() {
/*		settCost(cPUFactor);
		setmCost(memoryFactor);
		setArg3(parallelFactor);
		setArg4(0);
		setArg5(0);
		setArg6(0);*/
		setVersionString("");
		setKdf(this);
//		setParameters(cPUFactor, memoryFactor, parallelFactor);
	}

	@Override
	public byte[] performKDF(byte[] pswMaterial) {	
		
		if (KeyDerivation.getSalt() == null) {
			new UnexpectedValueException("byte[]", "Keyderivation.getSalt()","is null").printDescription();
			throw new IllegalArgumentException("Missing salt");
		} else {
			Comparator.checkNullVector(KeyDerivation.getSalt());
			Comparator.checkNullVector(pswMaterial);
		}

		if (KeyDerivation.getSalt().length < 16) {
			TestLog.e(getClass(), "Warning: Scrypt: short salt.");
		}
		if(KeyDerivation.getSalt().length < 8) {
			TestLog.e(getClass(), "Srypt: salt too short");
			throw new IllegalArgumentException("Scrypt - invalid salt size");
		}

		//long start =  System.currentTimeMillis(); // Startpunkt	
		byte[] keyMaterial = null;
		try {
		    keyMaterial = SCrypt.generate(pswMaterial, KeyDerivation.getSalt(), 
		    		gettCost(), getmCost(), getArg3(), 64);			
		} catch (Exception e) {
			TestLog.e(getClass(), "ScryptKDF Exception.");
			e.printStackTrace();
		}
		//TestLog.o(getClass(), "Scrypt: " + cPUFactor + " iterations, " + memoryFactor + " memory factor" );
		//printInfos(true);
		
		keyMaterial = adjustKeyMaterial(keyMaterial);
//Help.printBytes("keyMaterial", keyMaterial);
		return keyMaterial;
	}

	@Override
	public String getName() {
		return "Scrypt";
	}

	/**
	 * @return the memoryFactor
	 */
	public static int getMemoryFactor() {
		return memoryFactor;
	}

	/**
	 * @param _memoryFactor the memoryFactor to set
	 */
	public static void setMemoryFactor(int _memoryFactor) {
		memoryFactor = _memoryFactor;
	}

	/**
	 * @return the cPUFactor
	 */
	public static int getcPUFactor() {
		return cPUFactor;
	}

	/**
	 * @param _cPUFactor the cPUFactor to set
	 */
	public static void setcPUFactor(int _cPUFactor) {
		cPUFactor = _cPUFactor;
	}

	/**
	 * @return the parallelFactor
	 */
	public static int getParallelFactor() {
		return parallelFactor;
	}

	/**
	 * @param _parallelFactor the parallelFactor to set
	 */
	public static void setParallelFactor(int _parallelFactor) {
		parallelFactor = _parallelFactor;
	}

	/**
	 * Get the required memory for the key derivation function. 
	 * Returns the required memory in MiB. 
	 * 
	 * @return	memory requirement in MiB
	 */
	@Override
	public int getMemoryRequirement() {
		return 128 * cPUFactor * memoryFactor / (1024 * 1024);
	}

	@Override
	public String getInfoString() {
		StringBuilder sb = new StringBuilder();
		sb.append(getName() + ", ");
		sb.append("r: " + getmCost() + ", ");
		sb.append("N: " + gettCost()+ ", ");
		sb.append("p: " + getArg3());
		return new String(sb);
	}

	@Override
	public synchronized int gettCost() {
		return cPUFactor;
	}

	@Override
	public synchronized int getmCost() {
		return memoryFactor;
	}

	@Override
	public synchronized int getArg3() {
		return parallelFactor;
	}

	@Override
	public int getArg4() {
		return 0;
	}

	@Override
	public int getArg5() {
		return 0;
	}

	@Override
	public int getArg6() {
		return 0;
	}

/*	@Override
	public synchronized void settCost(int _tCost) {
		cPUFactor = _tCost;
	}

	@Override
	public synchronized void setmCost(int _mCost) {
		memoryFactor = _mCost;
	}

	@Override
	public synchronized void setArg3(int _arg3) {
		parallelFactor = _arg3;
	}

	@Override
	public void setArg4(int _arg4) {}
	@Override
	public void setArg5(int _arg5) {}
	@Override
	public void setArg6(int _arg6) {} */
}
